//
// IDEMGMT.H - IDE management support definitions of the PnP64x
//			   Scsi Miniport driver.
//
//			   Copyright (c) 2001-2005  Silicon Image.  All rights reserved.
//
// Edit History:
//
//		Aug. 03, 2004	Edited by David Canfield
//						Bumped management interface version for SATA
//						controllers from 2.2 to 2.3 to reflect definition of
//						PCI Express capabilities flags in the
//						SATA_CONTROLLER_INFO_BLK structure.
//		Mar. 18, 2004	Edited by David Canfield
//						Defined bit witin Flags field of DEVICE_INFO_BLK
//						structures for Storage Enclosure Processor (SEP)
//						devices.
//		Sep. 16, 2003	Edited by David Canfield
//						Bumped management interface version used by parallel
//						ATA controllers from 1.2 to 1.3 to reflect addition
//						of pass-thru IOCTL, IOCTL_CMD_ATAPI_PASS_THROUGH.
//		Aug. 13, 2003	Edited by David Canfield
//						Bumped management interface version 2.1 to 2.2
//						to reflect addition of pass-thru IOCTL
//						IOCTL_CMD_ATAPI_PASS_THROUGH.
//		Jul. 18, 2003	Edited by David Canfield
//						Bumped management interface version 2.0 to 2.1
//						to reflect the addition of two new IOCTLs
//						IOCTL_CMD_MINIPORT_GPIO_CONTROL and
//						IOCTL_CMD_MINIPORT_GPIO_RW.
//
//		Mar. 12, 2003	Edited by David Canfield
//						Added interface version 2.0 definitions for
//						SATA controllers.
//
//		Mar. 17, 2000	Created by David Canfield
//

#ifndef _IDEMGMT_DOT_H
#define _IDEMGMT_DOT_H

#ifndef WINDOWS
#ifndef GUID_DEF
#define GUID_DEF
// Define event GUID structures for non-Windows platforms.
typedef struct _GUID {
        unsigned long  Data1;         unsigned short Data2;
        unsigned short Data3;         unsigned char  Data4[8];
} GUID;
#endif
#endif // WINDOWS

                                                                                  
// SRB_FUNCTION_IO_CONTROL signature value used for controller
// management requests.
#define IDE_MANAGEMENT_SIGNATURE	"CMD_IDE "


// If user didn't specify a particular management interface version
// to support then include support for all versions.
#ifndef IDE_MANAGEMENT_VERSION
#define _ALL_VERSIONS
#endif


// Define management information version to be supported by driver.
#if IDE_MANAGEMENT_VERSION == 2

// Set current management interface version used by SATA controllers.
#define IDE_MGMT_MAJOR_VER			2
#define IDE_MGMT_MINOR_VER			2

#else

// Set current management interface version used by parallel ATA controllers.
#define IDE_MGMT_MAJOR_VER			1
#define IDE_MGMT_MINOR_VER			3
#endif


#if (!defined(_ALL_VERSIONS) && (IDE_MGMT_MINOR_VER != IDE_MANAGEMENT_REVISION))
#error Management interface revision level mismatch.
//Please make sure you have implemented all the IOCTLs required by
//the current managment interface revision and that you have defined
//IDE_MANAGEMENT_REVISION correctly!
#endif


//save compiler's current structure member alignment value, then change
//the alignment value to 1
#pragma pack(push,packvalue,1)


#if IDE_MANAGEMENT_VERSION == 2  || defined(_ALL_VERSIONS)
// Following definitions apply to version 2 of the management interface
// used with serial ATA controllers.


// Define layout of the SATA Controller Information Block.
// NOTE: Any changes here must be reflected in the definition of the WMI
// class, SATA_Controller, contained within Pnp64x.mof.
typedef struct _SATA_CONTROLLER_INFO_BLK {
	ULONG	ChipType;			// 3112, 3512, etc...
	ULONG	ChipRevision;		// Controller chip revision level
	ULONG	DriverType;			// Standard or RAID
	ULONG	PCIStatus;			// PCI bus information flags (see below)
	ULONG	IrqNumber;			// IRQ assigned to controller
	ULONG	PciBusNum;			// PCI Bus Number
	ULONG	PciDevNum;			// PCI Device Number
	ULONG	ControllerId;		// uniquely identifies an instance of a
								// controller within a system
	USHORT	ChannelCount;		// Number of channels supported by controller
	USHORT	TargetsPerChannel;	// Number of devices per channel supported by
								// controller
	CHAR	ManagementPath[128];// This field is filled in by the driver with
								// the location of where the GUI should store
								// configuration changes for this controller
								// type.  For example under Windows, the driver
								// will store the name of the driver's Service
								// key in the registry, other OS'es will
								// probably store the path to a configuration
								// file here.
} SATA_CONTROLLER_INFO_BLK, *PSATA_CONTROLLER_INFO_BLK;

#define DRIVER_TYPE_STANDARD	1
#define DRIVER_TYPE_RAID		4

// PCI bus information flags
#define PCI_BUS_SPEED_33		0x00000001	// PCI bus running at 33 Mhz.
#define PCI_BUS_SPEED_66		0x00000002	// PCI bus running at 66 Mhz.
#define PCI_BUS_SPEED_100		0x00000004	// PCI bus running at 100 Mhz.
#define PCI_BUS_SPEED_133		0x00000008	// PCI bus running at 133 Mhz.
#define PCI_BUS_PCIX			0x00000100	// Controller is PCI-X capable and
											// is plugged into a PCI-X bus.
#define PCI_BUS_64BIT			0x00000200	// Controller has 64-bit data bus
											// and is plugged into a 64-bit
											// capable bus.
											// PCI Express negotiated link
											// widths:
#define PCI_EXP_LINK_WIDTH_1	0x00000400	//	1
#define PCI_EXP_LINK_WIDTH_2	0x00000800	//	2
#define PCI_EXP_LINK_WIDTH_4	0x00001000	//	4
#define PCI_EXP_LINK_WIDTH_8	0x00002000	//	8
#define PCI_EXP_LINK_WIDTH_12	0x00004000	//	12
#define PCI_EXP_LINK_WIDTH_16	0x00008000	//	16
#define PCI_EXP_LINK_WIDTH_32	0x00010000	//	32
#define PCI_EXP_LINK_SPEED_2_5	0x00020000	// Controller has PCI Express
											// interface running with a link
											// speed of 2.5 Gb/s.


// Define layout of the SATA Channel Information Block.  The number of
// instances of this data block supported by SATA controller is specified
// by the ChannelCount field of the SATA_CONTROLLER_INFO_BLK structure.
// NOTE: Any changes here must be reflected in the definition of the WMI
// class, SATA_ChannelInfo, contained within Pnp64x.mof.
typedef struct _SATA_CHANNEL_INFO {
	ULONG	ControllerId;		// Identifies the controller instance to which
								// this channel information is associated.
	ULONG	Flags;				// Channel specific flags (see below)
	USHORT	ChannelIndex;		// Zero-based index indicating which channel
								// of the controller the information in this
								// structure is for
	USHORT	ChannelSpeed;		// Channel speed (ex. 1 = Generation 1 speed)
								// Following fields are only valid if
								// CFLAGS_PM_PRESENT is set in the Flags field
	USHORT	PM_VendorId;		//	 Vendor Id of port multiplier
	USHORT	PM_DeviceId;		//	 Device Id of port multiplier
	USHORT	PM_PortCount;		//	 Number of fan-out ports supported by
								//	 port multiplier
	UCHAR	PM_SpecVersion;		//	 Version of port multiplier specification
								//	 port multiplier supports
	UCHAR	PM_DeviceRevision;	//	 Revision level of the port multiplier
								//	 device
} SATA_CHANNEL_INFO, *PSATA_CHANNEL_INFO;

#define CFLAGS_PM_PRESENT	0x0001	// Channel has a port multiplier device attached



// Define layout of the SATA Device Information Block.
// NOTE: Any changes here must be reflected in the definition of the WMI
// class, SATA_DeviceInfo, contained within Pnp64x.mof.
typedef struct _SATA_DEVICE_INFO_BLK {
	ULONG	Flags;				// Various device state/status flags (see below)
	ULONG	ControllerId;		// Identifies the controller instance to which
								// a device is attached.
	ULONG	MaxIntXferSize;		// Number of words device is configured to
								// transfer per interrupt during Read/Write
								// Multiple commands.
	USHORT	XferModeLimit;		// User specified maximum transfer mode
								// device may be configured for (see below)
	USHORT	ChannelSpeed;		// Channel speed (ex. 1 = Generation 1 speed)		
	UCHAR	PathId;				// Indicates the bus on which the device is
								// located
	UCHAR	TargetId;			// Contains the target Id of the device.
	UCHAR	XferMode;			// PIO/DMA/UDMA mode device is configured for
	UCHAR	LunCount;			// Number of Logical units (discs) supported
								// by device.
	USHORT	IdentifyData[256];	// Raw Identify Device data read from device
} SATA_DEVICE_INFO_BLK, *PSATA_DEVICE_INFO_BLK;



// Define structure passed along with IOCTL_CMD_MINIPORT_GPIO_CONTROL
// requests.  This request is sent to enable/configure or disable
// a controller's GPIO interface.
// This IOCTL was added in revision 2.1 of the management interface.
typedef struct _GPIO_CONTROL {
	UCHAR	Enable;					// Enable GPIO interface if non-zero
									// otherwise disable it
	UCHAR	IoMask;					// Each bit set in this mask configures
									// corresponding GPIO pin for I/O
									// operation, zero bits configure the
									// corresponding pin for output only
									// operation
} GPIO_CONTROL, *PGPIO_CONTROL;


// Define structure passed along with IOCTL_CMD_MINIPORT_GPIO_RW
// requests.  This request is sent to either read or write a
// controller's GPIO interface pins.
// This IOCTL was added in revision 2.1 of the management interface.
typedef struct _GPIO_READ_WRITE {
	UCHAR	ReadWriteFlag;			// If non-zero, indicates a GPIO write
									// operation otherwise a GPIO read
									// operation is performed
	UCHAR	Data;					// Contains the data to be written for a
									// write operation, or the data read for
									// for a read operation
	UCHAR	TransitionDetect;		// For a read operation this field will
									// contain a bitmap indicating which GPIO
									// pins have changed since last read.
									// This field is not used during a write
									// operation.
} GPIO_READ_WRITE, *PGPIO_READ_WRITE;

#endif // end #if IDE_MANAGEMENT_VERSION == 2  || defined(_ALL_VERSIONS)


#if IDE_MANAGEMENT_VERSION == 1  || defined(_ALL_VERSIONS)
// Following definitions apply to version 1 of the management interface
// used with parallel ATA controllers.

// Define layout of the 648 IDE Controller Information Block.
// NOTE: Any changes here must be reflected in the definition of the WMI
// class, CMD_IDE_Controller, contained within Pnp64x.mof.

typedef struct _CHANNEL_INFO {
	ULONG	Flags;		// Channel specific flags (see below)
	ULONG	IdeBase;	// Base I/O address assigned to channel's
						//  IDE registers
	ULONG	AltBase;	// Base I/O address assigned to channel's
						//  Alternate registers
	ULONG	DmaBase;	// Base I/O address assigned to channel's
						//  DMA registers
} CHANNEL_INFO, *PCHANNEL_INFO;

#define CFLAGS_MASTER_PRESENT	0x0001	// Channel has a master device attached
#define CFLAGS_SLAVE_PRESENT	0x0002	// Channel has a slave device attached
#define CFLAGS_80PIN			0x0004	// Channel has an 80 pin cable

typedef struct _CMD_648_CONTROLLER_BLK {
	ULONG	ChipType;		// 648-U, 649-U, etc...
	ULONG	ChipRevision;	// Controller chip revision level
	ULONG	DriverType;		// Standard or RAID
	ULONG	Status;			// Operational status
	ULONG	IrqNumber;		// IRQ assigned to controller
	ULONG	PciBusNum;		// PCI Bus Number
	ULONG	PciDevNum;		// PCI Device Number
	ULONG	ControllerId;	// uniquely identifies an instance of a controller
							// within a system
	CHANNEL_INFO	ChannelInfo[2]; // Configuration information for both
									// channels of controller.
} CMD_648_CONTROLLER_BLK, *PCMD_648_CONTROLLER_BLK;

#define DRIVER_TYPE_STANDARD	1
#define DRIVER_TYPE_RAID		4


// Define layout of the IDE/ATAPI Device Information Block.
// NOTE: Any changes here must be reflected in the definition of the WMI
// class, CMD_IDE_ATAPI_DeviceInfo, contained within Pnp64x.mof.
typedef struct _DEVICE_INFO_BLK {
	ULONG	Flags;				// Various device state/status flags (see below)
	ULONG	ControllerId;		// Identifies the controller instance to which
								// a device is attached.
	ULONG	MaxIntXferSize;		// Number of words device is configured to
								// transfer per interrupt during Read/Write
								// Multiple commands.
	USHORT	XferModeLimit;		// User specified maximum transfer mode
								// device may be configured for (see below)

	UCHAR	PathId;				// Indicates the bus on which the device is
								// located
	UCHAR	TargetId;			// Contains the target Id of the device.
	UCHAR	XferMode;			// PIO/DMA/UDMA mode device is configured for
	UCHAR	LunCount;			// Number of Logical units (discs) supported
								// by device.
	USHORT	IdentifyData[256];	// Raw Identify Device data read from device
} DEVICE_INFO_BLK, *PDEVICE_INFO_BLK;

#endif // end #if IDE_MANAGEMENT_VERSION == 1  || defined(_ALL_VERSIONS)


// WMI DEVICE_INFO_BLK flag definitions
#define DEVINFO_ATAPI_DEVICE	0x0001	// ATAPI device
#define DEVINFO_ATAPI_CHANGER	0x0002	// Device is an ATAPI changer
#define DEVINFO_PIO_MODE		0x0004	// Device configured for PIO operation
#define DEVINFO_SDMA_MODE		0x0008	// Device configured for Singleword DMA
#define DEVINFO_MDMA_MODE		0x0010	// Device configured for Multiword DMA
#define DEVINFO_UDMA_MODE		0x0020	// Device configured for Ultra DMA
#define DEVINFO_UNSUPPORTED		0x0040	// Device is too old (pre-ATA) to support
#define DEVINFO_SEP_DEVICE		0x0080	// Storage Enclosure Processor device

// WMI DEVICE_INFO_BLK XferModeLimit definitions
#define PIO_MODE0				0		// Device limited to PIO Mode 0
#define PIO_MODE1				1		// Device limited to PIO Mode 1
#define PIO_MODE2				2		// Device limited to PIO Mode 2
#define PIO_MODE3				3		// Device limited to PIO Mode 3
#define PIO_MODE4				4		// Device limited to PIO Mode 4

#define MWDMA_MODE0				5		// Device limited to MDMA Mode 0
#define MWDMA_MODE1				6		// Device limited to MDMA Mode 1
#define MWDMA_MODE2				7		// Device limited to MDMA Mode 2

#define UDMA_MODE0				8		// Device limited to Ultra DMA Mode 0
#define UDMA_MODE1				9		// Device limited to Ultra DMA Mode 1
#define UDMA_MODE2				10		// Device limited to Ultra DMA Mode 2
#define UDMA_MODE3				11		// Device limited to Ultra DMA Mode 3
#define UDMA_MODE4				12		// Device limited to Ultra DMA Mode 4
#define UDMA_MODE5				13		// Device limited to Ultra DMA Mode 5
#define UDMA_MODE6				14		// Device limited to Ultra DMA Mode 6

#define MODE_AUTO_SELECT		0xFFFF	// Device transfer mode selected
										// by driver


// Define format of version information returned for
// IOCTL_CMD_MINIPORT_GET_VERSION requests.
typedef struct _IDE_MGMT_VERSION {
	USHORT		Major;			// Major version number
	USHORT		Minor;			// Minor version number
} IDE_MGMT_VERSION, *PIDE_MGMT_VERSION;


// Define structure used to specify the location of an ATA/ATAPI
// device (relative to a controller).
typedef struct _DEVICE_ADDRESS {
	UCHAR		PathId;			// Path (bus) number of device
	UCHAR		TargetId;		// Target Id of device.
} DEVICE_ADDRESS, *PDEVICE_ADDRESS;


#if IDE_MANAGEMENT_VERSION == 2  || defined(_ALL_VERSIONS)
// Define structure used in version two of the management interface to
// retrieve information about the channels of a SATA controller.
typedef struct _SATA_CHANNEL_INFO_REQUEST {
	USHORT			InstanceIndex;	// Index of 1st ATA/ATAPI device to return
									// information about.
	USHORT			InstanceCount;	// Number of devices (starting with
									// InstanceIndex) to return information about.
	SATA_CHANNEL_INFO ChannelInfo;	// Start of an InstanceCount length array of
									// channel information structures.
} SATA_CHANNEL_INFO_REQUEST, *PSATA_CHANNEL_INFO_REQUEST;



// Define structure used in version two of the management interface to
// retrieve information about the SATA devices attached to a controller.
typedef struct _SATA_DEV_INFO_REQUEST {
	USHORT			InstanceIndex;	// Index of 1st ATA/ATAPI device to return
									// information about.
	USHORT			InstanceCount;	// Number of devices (starting with
									// InstanceIndex) to return information about.
	SATA_DEVICE_INFO_BLK DeviceInfo;// Start of an InstanceCount length array of
									// device information structures.
} SATA_DEV_INFO_REQUEST, *PSATA_DEV_INFO_REQUEST;
#endif 


#if IDE_MANAGEMENT_VERSION == 1  || defined(_ALL_VERSIONS)
// Define format of structure passed into/returned from
// IOCTL_CMD_MINIPORT_GET_DEVICE_INFO requests.
typedef struct _DEV_INFO_REQUEST {
	USHORT			InstanceIndex;	// Index of 1st ATA/ATAPI device to return
									// information about.
	USHORT			InstanceCount;	// Number of devices (starting with
									// InstanceIndex) to return information about.
	DEVICE_INFO_BLK	DeviceInfo[];	// Start of an InstanceCount length array of
									// device information structures.
} DEV_INFO_REQUEST, *PDEV_INFO_REQUEST;
#endif


// Define structure passed along with IOCTL_CMD_MINIPORT_CACHE_CONTROL
// requests.  This request is sent to change the cache control settings
// of an ATA/ATAPI device.
typedef struct _CACHE_CONTROL {
	UCHAR	PathId;					// Channel number of target device			
	UCHAR	TargetId;				// Channel relative ID of device
	UCHAR	CacheFlags;				// Flags indicating if device's write/read
									// caches are to be turned on or off.
} CACHE_CONTROL, *PCACHE_CONTROL;

// Define the cache control flags for IOCTL_CMD_MINIPORT_CACHE_CONTROL
#define	CACHE_CTRL_READ_ENABLE	0x01	// Enable read cache if set, else disable
#define CACHE_CTRL_WRITE_ENABLE	0x02	// Enable write cache if set, else disable



// Define structure passed along with IOCTL_CMD_ATAPI_PASS_THROUGH requests.
// This IOCTL provides a mechanism to send any ATA or ATAPI command to the
// specified device.  This IOCTL was added to versions 1.3 and later of the
// management interface for parallel ATA controllers and to versions 2.2 and
// later of the management interface for SATA controllers.
typedef struct _ATAPI_PASS_THROUGH {
	USHORT	Length;					// Specifies the length (in bytes) of this
									// structure (NOTE: Length does not include
									// the length of any data transfer buffer
									// that may be associated with the command
	USHORT	Flags;					// Various command control flags (see below)
	UCHAR	PathId;					// Controller channel on which target
									// device is located
	UCHAR	TargetId;				// Channel relative ID of device 
	UCHAR	Lun;					// Logical unit number within target device
	UCHAR	CdbLength;				// Specifies the length of the Packet
									// command data contained in the Cdb field
	ULONG	DataTransferLength;		// Indicates the maximum size in bytes
									// of data transferred by the command
	ULONG_PTR DataBufferOffset;		// Specifies the offset in bytes from the
									// beginning of this structure to the data
									// buffer
	struct {
		UCHAR	Features_Err;		// Features reg. (write), Error reg. (read)
		UCHAR	Sector_Cnt;			// Sector Count register
		UCHAR	Sector_Num;			// Sector Number register
		UCHAR	Cyl_Low;			// Cylinder Low register
		UCHAR	Cyl_High;			// Cylinder High register
		UCHAR	Dev_Head;			// Device/Head register
		UCHAR	Cmd_Status;			// Command reg. (write), Status reg. (read)
		UCHAR	Sec_Num_Exp;		// Sector Number register high order byte
		UCHAR	Cyl_Low_Exp;		// Cylinder Low register high order byte
		UCHAR	Cyl_High_Exp;		// Cylinder High register high order byte
		UCHAR	Features_Exp;		// Features register high order byte
		UCHAR	Sector_Cnt_Exp;		// Sector Count register high order byte
	} TaskFileImage;
	UCHAR	Cdb[16];				// Contains the SCSI command to be sent
									// when performing an ATAPI Packet command
} ATAPI_PASS_THROUGH, *PATAPI_PASS_THROUGH;

#define ATAPI_PASS_THRU_DATA_OUT	0x00000001	// Data direction: TO device
#define ATAPI_PASS_THRU_DATA_IN		0x00000002	// Data direction: FROM device
#define ATAPI_PASS_THRU_48BIT_CMD	0x00000004	// Command uses extended
												// taskfile registers
#define ATAPI_PASS_THRU_PACKET_CMD	0x00000008	// Use ATAPI Packet protocol
												// to send command
#define ATAPI_PASS_THRU_USE_DMA		0x00000010	// Command requires the use
												// of DMA to perform data
												// transfer



// Define format of data buffer passed along with
// IOCTL_CMD_MINIPORT_WIN9X_POWER_NOTIFICATION
typedef enum _POWER_CHANGE_CODE {
	PCCStandby,						// system is going into standby
	PCCResume						// system is coming out of standby
} POWER_CHANGE_CODE;


// Define the signature of the function called to inform miniport driver
// of Win9x system power state changes.
// NOTE: The miniport driver expects interrupts to be disabled before
//		 this function is called!
typedef void (*POWER_NOTIFY)(IN PVOID DeviceExt,
							 IN POWER_CHANGE_CODE PwrChgCode);

// Define layout of structure passed along with
// IOCTL_CMD_MINIPORT_GET_WIN9X_POWER_NOTIFY_FUNCTION
typedef struct _POWER_FUNCTION_INFO {
	POWER_NOTIFY	PowerNotify;	// Pointer to miniport's power
									// notification function.
	PVOID			DeviceExt;		// Pointer to miniport's per controller
									// instance data.
} POWER_FUNCTION_INFO, *PPOWER_FUNCTION_INFO;


// Define event severity levels
typedef enum _EVENT_SEVERITY {
	SeverityInfo = 1,				// event is informational
	SeverityWarn,					// event is a warning
	SeverityError					// event is an error
} EVENT_SEVERITY;

// Define the maximum space available within an event packet
// for the "event specific" information.
#define MAX_EVENT_DATA_SIZE		44

// Define layout of event data reported to IOCTL based
// management applications.
typedef struct _EVENT_DATA {
	GUID			EventGuid;		// GUID that identifies this event
	ULONG			InstanceIdx;	// indicates which instance of the
									// event is being reported
	ULONG			Size;			// size of the following event data
	LARGE_INTEGER	Time;			// system time event occured
	ULONG			ControllerId;	// Identifies the controller instance
									// (within a system) that generated the
									// event.
	ULONG			SeqNumber;		// event sequence number
	ULONG			Severity;		// event severity level
	UCHAR			Data[MAX_EVENT_DATA_SIZE];	// data associated with event
} EVENT_DATA, *PEVENT_DATA;

// Define the layout of event data returned to a management application.
typedef struct _EVENT_HEADER {
	ULONG		EventCount;				// number of events that follow
	EVENT_DATA	Events[1];				// array of one or more event items
} EVENT_HEADER, *PEVENT_HEADER;

// Define layout of the data contained in the Ultra DMA mode downgrade
// event.
// NOTE: Any changes here must be reflected in the definition of the WMI
// class, CMD_IDE_UltraModeDowngradeEvent, contained within Pnp64x.mof.
typedef struct _ULTRA_DOWNGRADE_EVENT {
	UCHAR			PathId;		// indicates on which bus the downgraded
								// device is located
	UCHAR			TargetId;	// contains the target Id of the downgraded
								// device
	UCHAR			PreviousMode; // contains the Ultra DMA mode in which the
								// device was operating
	UCHAR			CurrentMode;// contains the new Ultra DMA mode selected by
								// the driver for the device
} ULTRA_DOWNGRADE_EVENT, *PULTRA_DOWNGRADE_EVENT;

// Define layout of the data contained in the new device detected,
// device removed/failed, and device changed events.
// NOTE: Any changes here must be reflected in the definition of the WMI
// classes CMD_IDE_DeviceChangeEvent, CMD_IDE_NewDeviceEvent, and
// CMD_IDE_DeletedDeviceEvent contained within Pnp64x.mof.
typedef struct _DEVICE_CHANGE_EVENT {
	UCHAR			PathId;		// indicates on which bus the changed
								// device is located
	UCHAR			TargetId;	// contains the target Id of the changed
								// device
	UCHAR			Model[40];	// contains the model string of the new
								// device, the deleted/failed device, or
								// the previous device when this structure
								// is used to report a new device detected,
								// device removed, or device changed event
								// respectively.
} DEVICE_CHANGE_EVENT, *PDEVICE_CHANGE_EVENT;


// Define flash memory access primitive opcodes.
typedef enum _FLASH_OPCODE {
	FlashWriteByte,
	FlashWriteBuffer,
	FlashReadBuffer,
	FlashDelay,
	FlashPollByte,
	FlashToggleWait
} FLASH_OPCODE;


typedef struct _FLASH_WRITE_BYTE {
	ULONG	Address;			// flash address to write
	UCHAR	ByteValue;			// value to write at address
} FLASH_WRITE_BYTE, *PFLASH_WRITE_BYTE;

typedef struct _FLASH_READ_WRITE_BUFFER {
	ULONG	StartAddr;			// starting address within flash to write/read
	ULONG	ByteCount;			// number of bytes to transfer to/from flash
	ULONG	BufferOffset;		// offset to the source/destination buffer for
								// data written/read from the flash
								// NOTE: Flash write/read buffer area begins
								// right after the last primitive in the
								// Primitives array
} FLASH_READ_WRITE_BUFFER, *PFLASH_READ_WRITE_BUFFER;

typedef struct _FLASH_DELAY {
	ULONG	DelayCount;			// number of microseconds to delay, valid
								// range for DelayCount is:
								// 0 < DelayCount < 100000
} FLASH_DELAY, *PFLASH_DELAY;

typedef struct _FLASH_POLL_FOR_BYTE {
	ULONG	Address;			// address within flash to poll
	ULONG	PollCount;			// number of times to poll flash address
								// NOTE: the driver inserts a 1 us. between
								// successive poll operations so that the
								// polling interval will be fairly consistent
								// regardless of system speed.
	UCHAR	ByteValue;			// value to poll for
} FLASH_POLL_FOR_BYTE, *PFLASH_POLL_FOR_BYTE;

typedef struct _FLASH_TOGGLE_WAIT {
	ULONG	Address;			// address within flash to monitor
	ULONG	PollCount;			// number of times to poll flash address
								// NOTE: the driver inserts a 1 us. between
								// successive poll operations so that the
								// polling interval will be fairly consistent
								// regardless of system speed.
	UCHAR	BitMask;			// mask representing bit(s) to check for
								// toggling
} FLASH_TOGGLE_WAIT, *PFLASH_TOGGLE_WAIT;


// Define layout of a single flash access primitive.
typedef struct _FLASH_PRIMITIVE {
	FLASH_OPCODE	Opcode;
	union {
		// NOTE: List largest union member 1st.  This will avoid a "too many
		//		 initializers" error when declaring initialized arrays of
		//		 flash primitives.
		FLASH_READ_WRITE_BUFFER	FlashReadWriteBuffer;
		FLASH_WRITE_BYTE		FlashWriteByte;
		FLASH_DELAY				FlashDelay;
		FLASH_POLL_FOR_BYTE		FlashPollByte;
		FLASH_TOGGLE_WAIT		FlashToggleWait;
	};
} FLASH_PRIMITIVE, *PFLASH_PRIMITIVE;


// Define format of structure passed into/returned from
// IOCTL_CMD_MINIPORT_FLASH_ACCESS requests.
typedef struct _FLASH_ACCESS {
	ULONG	PrimitiveCount;
	ULONG	IoBufferSize;
#ifdef WINDOWS
	union {
		FLASH_PRIMITIVE	Primitives[];
		UCHAR			IoBuffer[];
	};
#else
                FLASH_PRIMITIVE *Primitives;
                UCHAR           *IoBuffer;
#endif
} FLASH_ACCESS, *PFLASH_ACCESS;


// Define layout of information passed at SRB->DataBuffer for
// controller management requests.
typedef struct _SRB_IDE_IOCTL_BLOCK {
    SRB_IO_CONTROL	Header;
	union {
		IDE_MGMT_VERSION			Version;

#ifdef _ALL_VERSIONS
		CMD_648_CONTROLLER_BLK		CntrlrInfo;
		SATA_CONTROLLER_INFO_BLK	SataCntrlrInfo;
		SATA_CHANNEL_INFO_REQUEST	SataChanInfoRequest;
		DEV_INFO_REQUEST			DevInfoRequest;
		SATA_DEV_INFO_REQUEST		SataDevInfoRequest;
		GPIO_CONTROL				GpioCtrlRequest;
		GPIO_READ_WRITE				GpioRWRequest;

#elif IDE_MANAGEMENT_VERSION == 2
		SATA_CONTROLLER_INFO_BLK	SataCntrlrInfo;
		SATA_CHANNEL_INFO_REQUEST	SataChanInfoRequest;
		SATA_DEV_INFO_REQUEST		SataDevInfoRequest;
		GPIO_CONTROL				GpioCtrlRequest;
		GPIO_READ_WRITE				GpioRWRequest;
#else
		CMD_648_CONTROLLER_BLK		CntrlrInfo;
		DEV_INFO_REQUEST			DevInfoRequest;
#endif

		DEVICE_ADDRESS				DeviceAddress;
		EVENT_HEADER				EventHeader;
		POWER_FUNCTION_INFO			PowerFunctionInfo;
		FLASH_ACCESS				FlashAccess;
		CACHE_CONTROL				CacheControl;
		ATAPI_PASS_THROUGH			AtapiPassThrough;
	    UCHAR						DataBuffer[1];
	};
} SRB_IDE_IOCTL_BLOCK, *PSRB_IDE_IOCTL_BLOCK;

// Define miniport IOCTL status codes.
#define IOCTL_IDE_STATUS_SUCCESS	0	// Operation completed successfully
#define IOCTL_IDE_STATUS_REBOOT		1	// Reboot required for change to take
										// effect
#define IOCTL_IDE_STATUS_INVLD		2	// Invalid parameter
#define IOCTL_IDE_STATUS_BUFFER		3	// caller's buffer is too small
#define IOCTL_IDE_STATUS_NODEVICE	4	// Specified device not detected
#define IOCTL_IDE_STATUS_DEVICECHG	5	// Device insertion/removal detected
#define IOCTL_IDE_STATUS_TIMEOUT	6	// The requested operation timed out
#define IOCTL_IDE_FEATURE_NOT_AVAIL	7	// Feature not available
#define IOCTL_IDE_NOT_ENABLED		8	// The requested operation must first
										// be enabled
#define IOCTL_IDE_STATUS_ERROR		255	// General catch-all error status


/* Macro.Properties from Java Code */

#define IOCTL_CMD_MINIPORT_GET_CHANNEL_INFO 270368
#define IOCTL_CMD_MINIPORT_GET_NUM_OF_CNTRLS 1002
#define IOCTL_CMD_MINIPORT_GET_VERSION 270336
#define IOCTL_CMD_MINIPORT_GET_CONTROLLER_INFO 270340
#define IOCTL_CMD_MINIPORT_GET_CONTROLLER_FILEINFO 1000
#define IOCTL_CMD_MINIPORT_GET_DEVICE_INFO 270344
#define IOCTL_CMD_MINIPORT_GET_EVENT_DATA 270348
#define IOCTL_CMD_MINIPORT_CHECK_DEVICE_PRESENT 270352
#define IOCTL_CMD_MINIPORT_CACHE_CONTROL 270364
#define IOCTL_CMD_RIO_GET_VERSION 270336
#define IOCTL_CMD_RIO_CREATE_ARRAY 270340
#define IOCTL_CMD_RIO_DELETE_ARRAY 270344
#define IOCTL_CMD_RIO_ADD_MEMBER 270348
#define IOCTL_CMD_RIO_REMOVE_MEMBER 270352
#define IOCTL_CMD_RIO_INITIALIZE_ARRAY 270356
#define IOCTL_CMD_RIO_REBUILD_MEMBER 270360
#define IOCTL_CMD_RIO_GET_ARRAY_INFO 270372
#define IOCTL_CMD_RIO_RESET_BUS 270376
#define IOCTL_CMD_RIO_GET_SPARE_INFO 270380
#define IOCTL_CMD_MINIPORT_ENABLE_SPARE 270384
#define IOCTL_CMD_RIO_POLLING_STATUS 270388
#define IOCTL_CMD_RIO_REBUILD_RATE 270392
#define IOCTL_CMD_RIO_GET_IDENTIFY 270396
#define IOCTL_CMD_RIO_CREATE_SPARE 270400
#define IOCTL_CMD_RIO_DELETE_SPARE 270404
#define IOCTL_CMD_VSD_GET_NUM_OF_CNTRLS 270340
//#define IOCTL_SCSI_MINIPORT_RETURN_STATUS 1770758
#define IOCTL_CMD_MINIPORT_SET_DEVICE_MODE 1001

// restore compiler's structure alignment value
#pragma pack(pop, packvalue)

#ifdef OLD
// Device I/O Control code used to retrieve the version of the management
// information provided by the miniport driver.
#define IOCTL_CMD_MINIPORT_GET_VERSION \
	CTL_CODE(FILE_DEVICE_CONTROLLER, 0x800, METHOD_BUFFERED, FILE_ANY_ACCESS)

// Device I/O Control code used to retrieve the controller configuration
// information from the miniport driver.
#define IOCTL_CMD_MINIPORT_GET_CONTROLLER_INFO \
	CTL_CODE(FILE_DEVICE_CONTROLLER, 0x801, METHOD_BUFFERED, FILE_ANY_ACCESS)

// I/O Control code used to retrieve information from the miniport driver
// about one or more devices attached to a controller.
#define IOCTL_CMD_MINIPORT_GET_DEVICE_INFO \
	CTL_CODE(FILE_DEVICE_CONTROLLER, 0x802, METHOD_BUFFERED, FILE_ANY_ACCESS)

// I/O Control code used to retrieve the data for events that have occured.
#define IOCTL_CMD_MINIPORT_GET_EVENT_DATA \
	CTL_CODE(FILE_DEVICE_CONTROLLER, 0x803, METHOD_BUFFERED, FILE_ANY_ACCESS)

// I/O Control code used to cause driver to check to see that the specified
// device is still connected to the controller.
#define IOCTL_CMD_MINIPORT_CHECK_DEVICE_PRESENT \
	CTL_CODE(FILE_DEVICE_CONTROLLER, 0x804, METHOD_BUFFERED, FILE_ANY_ACCESS)

#endif //OLD

// I/O Control code used on Win9x systems to retrieve the address of the
// miniport's Win9x power change notification function.  The function will be
// called by the companion VXD to notify the driver about system power state
// changes (ie. going into or coming out of standby).
// NOTE: Since this IOCTL is only sent by the driver's companion VXD there
//		 is no need to create a GUID for it so that it can be sent using WMI.
#define IOCTL_CMD_MINIPORT_GET_WIN9X_POWER_NOTIFY_FUNCTION \
	CTL_CODE(FILE_DEVICE_CONTROLLER, 0x805, METHOD_BUFFERED, FILE_ANY_ACCESS)

// I/O Control code used to provide access to flash memory expansion BIOS
// device that may be attached to the controller.
#define IOCTL_CMD_MINIPORT_FLASH_ACCESS \
	CTL_CODE(FILE_DEVICE_CONTROLLER, 0x806, METHOD_BUFFERED, FILE_ANY_ACCESS)

#ifdef OLD
// I/O Control code used to change the cache settings of a specific
// ATA or ATAPI device.
#define IOCTL_CMD_MINIPORT_CACHE_CONTROL \
	CTL_CODE(FILE_DEVICE_CONTROLLER, 0x807, METHOD_BUFFERED, FILE_ANY_ACCESS)
#endif

#if IDE_MANAGEMENT_VERSION == 2  || defined(_ALL_VERSIONS)
// I/O Control code available when version 2 of the management interface is
// in use by the miniport driver to retrieve information about one or more
// of the controller's channels.
#ifdef OLD
#define IOCTL_CMD_MINIPORT_GET_CHANNEL_INFO \
	CTL_CODE(FILE_DEVICE_CONTROLLER, 0x808, METHOD_BUFFERED, FILE_ANY_ACCESS)
#endif //OLD

// I/O Control code available when version 2 of the management interface is
// in use by the miniport driver to configure a controller's GPIO port.
#define IOCTL_CMD_MINIPORT_GPIO_CONTROL \
	CTL_CODE(FILE_DEVICE_CONTROLLER, 0x809, METHOD_BUFFERED, FILE_ANY_ACCESS)

// I/O Control code available when version 2 of the management interface is
// in use by the miniport driver to access a controller's GPIO port.
#define IOCTL_CMD_MINIPORT_GPIO_RW \
	CTL_CODE(FILE_DEVICE_CONTROLLER, 0x80A, METHOD_BUFFERED, FILE_ANY_ACCESS)
#endif

//------------------- SHW / CEP ADDED
// Device I/O Control code used to retrieve the count of cards
#define IOCTL_CMD_MINIPORT_GET_NUM_OF_CNTRLS \
    1002
                                                                                                                       
// Java GUI uses this macro to retrieve driver file's information
#define IOCTL_CMD_MINIPORT_GET_CONTROLLER_FILEINFO \
    1000
                                                                                                                       
// Java GUI uses this macro to set the transfer mode of hard disk
#define IOCTL_CMD_MINIPORT_SET_DEVICE_MODE \
    1001


// I/O control code to send any ATA or ATAPI Packet command through the
// driver to a device.  This IOCTL was added to versions 1.3 and later of
// the management interface for parallel ATA controllers and to versions
// 2.2 and later of the management interface for SATA controllers.
#define IOCTL_CMD_ATAPI_PASS_THROUGH \
	CTL_CODE(FILE_DEVICE_CONTROLLER, 0x80B, METHOD_BUFFERED, FILE_ANY_ACCESS)


#ifdef RAID_INTERFACE
// Define guid that identifies the ATA controller data block type.
static const GUID CMD_CntrlrBlockGUID = {0xE9A4D776, 0xCD71, 0x4568,
							{0x8E, 0x17, 0x22, 0xDA, 0x14, 0xB7, 0xB3, 0x51}};

// Define guid that identifies the SATA controller data block type.
static const GUID SATA_CntrlrBlockGUID = {0xB052ACFB, 0x14A1, 0x4DBB,
							{0xAA, 0xF9, 0xB8, 0x05, 0xC3, 0x10, 0x25, 0x49}};

// Define guid that identifies the SATA controller channel data block type.
static const GUID SATA_ChannelBlockGUID = {0x337B0334, 0x3C63, 0x4DDA,
							{0xA9, 0xAC, 0x28, 0x21, 0x8C, 0xC2, 0x15, 0xCD}};

// Define guid that identifies the SATA device information
// data block type.
static const GUID SATA_DeviceInfoBlockGUID = {0x0C451AA4, 0x0771, 0x4F19,
							{0x9D, 0xB1, 0xD6, 0x0F, 0xFC, 0x4F, 0xB7, 0x27}};

// Define guid that identifies the ATA/ATAPI device information
// data block type.
static const GUID DeviceInfoBlockGUID = {0x0FC4BB31, 0x25F7, 0x11D4,
							{0xAC, 0x5D, 0x00, 0x10, 0x4B, 0x2E, 0x9B, 0xBD}};

// Define guid that identifies the Ultra DMA mode change event block type.
static const GUID UltraModeEventGUID = {0x650CC120, 0x94B0, 0x46EF,
							{0x9C, 0x61, 0x05, 0x20, 0x19, 0x9E, 0x32, 0x32}};

// Define guid that identifies the device changed event block type.
static const GUID DeviceChangedEventGUID = {0x629096C0, 0xFFB7, 0x4998,
							{0x8D, 0xFA, 0x4C, 0x39, 0x23, 0xDB, 0xBE, 0xDC}};

// Define guid that identifies the new device detected event block type.
static const GUID NewDeviceEventGUID = {0x392545EB, 0xC108, 0x46C5,
							{0xA4, 0x5F, 0xAB, 0xBB, 0xEC, 0xDC, 0xA7, 0x62}};

// Define guid that identifies the device deleted/failed event block type.
static const GUID DeletedDeviceEventGUID = {0x90E49A68, 0x52C8, 0x47A9,
							{0xB5, 0x0B, 0x4D, 0x2C, 0x6F, 0xA9, 0x4B, 0x4B}};

// Define guid that identifies the unable to configure transfer mode for
// device event block type.
static const GUID UnconfigurableDeviceEventGUID = {0x1A235E09, 0x4C7C, 0x49F9,
							{0xB9, 0x47, 0x04, 0xEB, 0x41, 0xA9, 0xD8, 0xBA}};
#endif // RAID_INTERFACE

#endif	// ifndef _IDEMGMT_DOT_H
