//****************************************************************************
//  Linux Host Driver for SiI SATA RAID Adapters
//
//  debug.h  (debug support)
// 
//
// Copyright (c) 2005  Silicon Image.  All rights reserved.
//
// Creator: Vu Le
//
//****************************************************************************

#ifndef SI_DEBUG_
#define SI_DEBUG_


// DEBUG MESSAGE DEFINES
//#define WRPR_DBG    // wrapper debug messages
//#define DBG         // sii driver library debug messages
//#define RAID_DBG  // sii legacy raid debug messages


#if defined(WRPR_DBG) || defined(DBG) || defined(RAID_DBG)
#define PRINT_OFF 0 // debug printing enabled
#else
#define PRINT_OFF 1 // all debug printing disabled (turned off)
#endif



extern asmlinkage void sii_print(const char *format, ...);
extern asmlinkage void sii_printd(const char *format, ...);


// Level 1 is error, 2 is warning
// Level 3 is special to debug sections of code (Development only)
#define   ERROR_CODE   		1       // debug
#define   WARN_CODE   		2       // debug
#define   ENG_DEBUG_CODE   	3       // debug

//#define   DebugPrintLevel  WARN_CODE
#define   _DebugPrintLevel_  WARN_CODE

#ifdef  FULL_PCI_TRACE  // Currently only works with ONE CHIPSET
void *PCI_Software_Reg = 0x0;
                                                                                
// Display a combination of  0xAC..line  (.. is first byte of function name)
VOID __inline__ PCI_TRACE0( PCHAR func, INT line){
                                                                                
 u32 thirtytwobits = (0xAC << 24) + ((*func) << 16 & 0xff) + (line & 0xffff);
                                                                                
 if (PCI_Software_Reg) {
   sii_print(" -- addr=%lx,dat=%x -- ",
            (POINTER) PCI_Software_Reg,(INT) (thirtytwobits))
   WriteRegisterUlong(PCI_Software_Reg,thirtytwobits);
 }
                                                                                
}
#endif
//
// Beautification macros
//

#define DPRINT(x...) 	DEBUG(x...)
#define DbgPrint(x...) 	DEBUG(x...)

// Enable warning and error messages (PRINT)
#define PRINT

#ifdef  FULL_PCI_TRACE
#define MSG(x...) {PCI_TRACE0(__FUNCTION__,__LINE__);sii_print(x);} // Always OUTPUT
#else
#define MSG(x...) {sii_print(x);} // Always OUTPUT
#endif

#ifdef  PRINT_DEBUG
#ifdef FULL_PCI_TRACE
#define DEBUG(x...) {PCI_TRACE0(__FUNCTION__,__LINE__);	\
	if(DebugPrintLevel >= ENG_DEBUG_CODE) {sii_print(x);}}
#else
#define DEBUG(x...) {if(DebugPrintLevel >= ENG_DEBUG_CODE) {sii_print(x);}}
#endif
#else
#define DEBUG(x...)
#endif

#ifdef PRINT

#ifdef  FULL_PCI_TRACE
#define WARN_MSG(x...) {PCI_TRACE0(__FUNCTION__,__LINE__);	\
	if(DebugPrintLevel >= WARN_CODE) {sii_print(x);}}
#else
#define WARN_MSG(x...) {if(DebugPrintLevel >= WARN_CODE) {sii_print(x);} }
#endif

#ifdef  FULL_PCI_TRACE
#define ERR_MSG(x...)  {PCI_TRACE0(__FUNCTION__,__LINE__);	\
	if(_DebugPrintLevel_ >= ERROR_CODE) {sii_print(x);}}
#else
#define ERR_MSG(x...)  {if(DebugPrintLevel >= ERROR_CODE) {sii_print(x); }}
#endif

#else
#define WARN_MSG(x...)
#define ERR_MSG(x...)
#endif

#define ERROR_MSG   ERR_MSG
#define WARNING_MSG WARN_MSG

// Development only
#define DEV1_MSG(x...) {if((DebugPrintLevel == ENG_DEBUG_CODE) ||	\
	(DebugPrintLevel > WARN_CODE)) {sii_print(x);}}  // for development only
#define DEV2_MSG(x...)
#define DEV3_MSG(x...)
#define DEV4_MSG(x...)

#ifdef WRPR_DBG
 #define WDBG_MSG(x)    sii_printd x ;
#else
 #define WDBG_MSG(x)
#endif

#ifdef RAID_DBG
 extern asmlinkage void     DebugMsg		(const char *, ...);
 #define BASIC_DEBUG_MSG(x)             DebugMsg x ;
 #define        DEBUG_RAID_MSG(x)       DebugMsg x ;
 #define        DEBUG_RAID_WARN_MSG(x)  DebugMsg x ;
 #define        DEBUG_RAID_ERR_MSG(x)   DebugMsg x ;
// #define        RAY_DEBUG_MSG(x)        DebugMsg x ;
 #define        RAY_DEBUG_MSG(x)
 #define        KdPrint(x)              DebugMsg x ;
#else
 #define BASIC_DEBUG_MSG(_x_)
 #define        DEBUG_RAID_MSG(x)
 #define        DEBUG_RAID_WARN_MSG(x)
 #define        DEBUG_RAID_ERR_MSG(x)
 #define        RAY_DEBUG_MSG(x)
 #define        KdPrint(_x_)
#endif

#endif 	// SI_DEBUG_
