//****************************************************************************
//	Linux Host Driver for SiI SATA RAID Adapters
//
// sii_Wscsi.h - Include file
//
// Copyright (c) 2005  Silicon Image.  All rights reserved.
// Creator: Vu Le
//
//****************************************************************************
#ifndef _SII_WSCSI
#define _SII_WSCSI

//#define siStatic static

//
// Notification Event Types
//
typedef enum _SCSI_NOTIFICATION_TYPE
{
    RequestComplete,
    NextRequest,
    NextLuRequest,
    ResetDetected,
    CallDisableInterrupts,
    CallEnableInterrupts,
    RequestTimerCall,
    BusChangeDetected,     /* New */       
    WMIEvent,
    WMIReregister
} SCSI_NOTIFICATION_TYPE, *PSCSI_NOTIFICATION_TYPE;

#define	SP_RETURN_FOUND		0
#define	SP_RETURN_ERROR		1
#define	SP_RETURN_BAD_CONFIG	2	
#define	SP_RETURN_NOT_FOUND	3


#ifndef _SII_WINSIF
#ifndef BUS_DATA_TYPE 
// not defined in <sii_Winsif.h>
// Define types of bus information.
typedef enum _BUS_DATA_TYPE {
        ConfigurationSpaceUndefined = -1,
        Cmos,
        EisaConfiguration,
        Pos,
        CbusConfiguration,
        PCIConfiguration,
        VMEConfiguration,
        NuBusConfiguration,
        PCMCIAConfiguration,
        MPIConfiguration,
        MPSAConfiguration,
        PNPISAConfiguration,
        SgiInternalConfiguration,
        MaximumBusDataType
} BUS_DATA_TYPE, *PBUS_DATA_TYPE;
#endif
#endif
                                                                                
#ifndef INTERFACE_TYPE
typedef enum _INTERFACE_TYPE {
    InterfaceTypeUndefined = -1,
    Internal,
    Isa,
    Eisa,
    MicroChannel,
    TurboChannel,
    PCIBus,
    VMEBus,
    NuBus,
    PCMCIABus,
    CBus,
    MPIBus,
    MPSABus,
    ProcessorInternal,
    InternalPowerBus,
    PNPISABus,
    PNPBus,
    MaximumInterfaceType
} INTERFACE_TYPE, *PINTERFACE_TYPE;
#endif

//#define PACKED                  __attribute__((packed))

typedef union _CDB {

    //
    // Generic 6-Byte CDB
    //

    struct _CDB6GENERIC {
       UCHAR  OperationCode;
       UCHAR  Immediate : 1;
       UCHAR  CommandUniqueBits : 4;
       UCHAR  LogicalUnitNumber : 3;
       UCHAR  CommandUniqueBytes[3];
       UCHAR  Link : 1;
       UCHAR  Flag : 1;
       UCHAR  Reserved : 4;
       UCHAR  VendorUnique : 2;
    } CDB6GENERIC, *PCDB6GENERIC;

    //
    // Standard 6-byte CDB
    //

    struct _CDB6READWRITE {
        UCHAR OperationCode;
        UCHAR LogicalBlockMsb1 : 5;
        UCHAR LogicalUnitNumber : 3;
        UCHAR LogicalBlockMsb0;
        UCHAR LogicalBlockLsb;
        UCHAR TransferBlocks;
        UCHAR Control;
    } CDB6READWRITE, *PCDB6READWRITE;

    //
    // SCSI Inquiry CDB
    //

    struct _CDB6INQUIRY {
        UCHAR OperationCode;
        UCHAR Reserved1 : 5;
        UCHAR LogicalUnitNumber : 3;
        UCHAR PageCode;
        UCHAR IReserved;
        UCHAR AllocationLength;
        UCHAR Control;
    } CDB6INQUIRY, *PCDB6INQUIRY;

    struct _CDB6VERIFY {
        UCHAR OperationCode;
        UCHAR Fixed : 1;
        UCHAR ByteCompare : 1;
        UCHAR Immediate : 1;
        UCHAR Reserved : 2;
        UCHAR LogicalUnitNumber : 3;
        UCHAR VerificationLength[3];
        UCHAR Control;
    } CDB6VERIFY, *PCDB6VERIFY;
#ifndef __DRIVER__
    struct _START_STOP {
        UCHAR OperationCode;
        UCHAR Immediate : 1;
        UCHAR Reserved1 : 4;
        UCHAR LogicalUnitNumber : 3;
        UCHAR Reserved2[2];
	UCHAR Start: 1;
        UCHAR LoadEject: 1;
	UCHAR Reserved3: 6;
        UCHAR Control;
    } START_STOP, *PSTART_STOP;
#endif    
	

    //
    // Standard 10-byte CDB

    struct _CDB10 {
        UCHAR OperationCode;
        UCHAR RelativeAddress : 1;
        UCHAR Reserved1 : 2;
        UCHAR ForceUnitAccess : 1;
        UCHAR DisablePageOut : 1;
        UCHAR LogicalUnitNumber : 3;
        UCHAR LogicalBlockByte0;
        UCHAR LogicalBlockByte1;
        UCHAR LogicalBlockByte2;
        UCHAR LogicalBlockByte3;
        UCHAR Reserved2;
        UCHAR TransferBlocksMsb;
        UCHAR TransferBlocksLsb;
        UCHAR Control;
    } CDB10, *PCDB10;

    //
    // Standard 12-byte CDB
    //

    struct _CDB12 {
        UCHAR OperationCode;
        UCHAR RelativeAddress : 1;
        UCHAR Reserved1 : 2;
        UCHAR ForceUnitAccess : 1;
        UCHAR DisablePageOut : 1;
        UCHAR LogicalUnitNumber : 3;
        UCHAR LogicalBlock[4];      // [0]=MSB, [3]=LSB
        UCHAR TransferLength[4];    // [0]=MSB, [3]=LSB
        UCHAR Reserved2;
        UCHAR Control;
    } CDB12, *PCDB12;

    //
    // Mode select
    //
#if 0
    struct _MODE_SELECT {
        UCHAR OperationCode;    // 0x15 - SCSIOP_MODE_SELECT
        UCHAR SPBit : 1;
        UCHAR Reserved1 : 3;
        UCHAR PFBit : 1;
        UCHAR LogicalUnitNumber : 3;
        UCHAR Reserved2[2];
        UCHAR ParameterListLength;
        UCHAR Control;
    } MODE_SELECT, *PMODE_SELECT;
#endif                                                                                
    struct _MODE_SELECT10 {
        UCHAR OperationCode;    // 0x55 - SCSIOP_MODE_SELECT10
        UCHAR SPBit : 1;
        UCHAR Reserved1 : 3;
        UCHAR PFBit : 1;
        UCHAR LogicalUnitNumber : 3;
        UCHAR Reserved2[5];
        UCHAR ParameterListLength[2];
        UCHAR Control;
    } MODE_SELECT10, *PMODE_SELECT10;

    struct _LOCATE {
        UCHAR OperationCode;    // 0x2B - SCSIOP_LOCATE
        UCHAR Immediate : 1;
        UCHAR CPBit : 1;
        UCHAR BTBit : 1;
        UCHAR Reserved1 : 2;
        UCHAR LogicalUnitNumber : 3;
        UCHAR Reserved3;
        UCHAR LogicalBlockAddress[4];
        UCHAR Reserved4;
        UCHAR Partition;
        UCHAR Control;
    } LOCATE, *PLOCATE;
                                                                                
    struct _LOGSENSE {
        UCHAR OperationCode;    // 0x4D - SCSIOP_LOG_SENSE
        UCHAR SPBit : 1;
        UCHAR PPCBit : 1;
        UCHAR Reserved1 : 3;
        UCHAR LogicalUnitNumber : 3;
        UCHAR PageCode : 6;
        UCHAR PCBit : 2;
        UCHAR Reserved2;
        UCHAR Reserved3;
        UCHAR ParameterPointer[2];
        UCHAR AllocationLength[2];
        UCHAR Control;
    } LOGSENSE, *PLOGSENSE;
                                                                                
    struct _LOGSELECT {
        UCHAR OperationCode;    // 0x4C - SCSIOP_LOG_SELECT
        UCHAR SPBit : 1;
        UCHAR PCRBit : 1;
        UCHAR Reserved1 : 3;
        UCHAR LogicalUnitNumber : 3;
        UCHAR Reserved : 6;
        UCHAR PCBit : 2;
        UCHAR Reserved2[4];
        UCHAR ParameterListLength[2];
        UCHAR Control;
    } LOGSELECT, *PLOGSELECT;
    
} /*PACKED*/ CDB, *PCDB;

#ifndef KINTERRUPT_MODE
typedef enum _KINTERRUPT_MODE
{
    LevelSensitive,
    Latched
}KINTERRUPT_MODE;
#endif

typedef enum _DMA_SPEED
{
    Compatible,
    TypeA,
    TypeB,
    TypeC,
    TypeF,
    MaximumDmaSpeed
} DMA_SPEED, *PDMA_SPEED;

typedef enum _DMA_WIDTH
{
    Width8Bits,
    Width16Bits,
    Width32Bits,
    MaximumDmaWidth
} DMA_WIDTH, *PDMA_WIDTH;

typedef struct _ACCESS_RANGE
{
    SCSI_PHYSICAL_ADDRESS	RangeStart;
    ULONG			RangeLength;
    BOOLEAN			RangeInMemory;
} ACCESS_RANGE, *PACCESS_RANGE;


typedef struct _PORT_CONFIGURATION_INFORMATION
{
    ULONG  Length;
    ULONG  SystemIoBusNumber;
    INTERFACE_TYPE  AdapterInterfaceType;
    ULONG  BusInterruptLevel;
    ULONG  BusInterruptVector;
    KINTERRUPT_MODE  InterruptMode;
    ULONG  MaximumTransferLength;
    ULONG  NumberOfPhysicalBreaks;
    ULONG  AlignmentMask;
    ULONG  NumberOfAccessRanges;
    ACCESS_RANGE  **AccessRanges;
    UCHAR  NumberOfBuses;
    UCHAR  InitiatorBusId[8];
    BOOLEAN  ScatterGather;
    BOOLEAN  Master;
    BOOLEAN  CachesData;
    BOOLEAN  AtdiskPrimaryClaimed;
    BOOLEAN  AtdiskSecondaryClaimed;
    BOOLEAN  Dma32BitAddresses;
    BOOLEAN  MapBuffers;
    BOOLEAN  NeedPhysicalAddresses;
    BOOLEAN  TaggedQueueing;
    BOOLEAN  AutoRequestSense;
    BOOLEAN  MultipleRequestPerLu;
    UCHAR  MaximumNumberOfTargets;
    ULONG  SlotNumber;
    ULONG  DeviceExtensionSize;
    ULONG  SrbExtensionSize;
    UCHAR  Dma64BitAddresses;
    BOOLEAN  WmiDataProvider;
} PACKED PORT_CONFIGURATION_INFORMATION,
*PPORT_CONFIGURATION_INFORMATION;

typedef enum _SCSI_ADAPTER_CONTROL_TYPE
{
    ScsiQuerySupportedControlTypes,
    ScsiStopAdapter,
    ScsiRestartAdapter,
    ScsiSetBootConfig,
    ScsiSetRunningConfig
} SCSI_ADAPTER_CONTROL_TYPE;

typedef enum _SCSI_ADAPTER_CONTROL_STATUS
{
    ScsiAdapterControlSuccess,
    ScsiAdapterControlUnsuccessful
} SCSI_ADAPTER_CONTROL_STATUS;

typedef asmlinkage BOOLEAN (*PHW_BUILDIO) (PVOID, PSCSI_REQUEST_BLOCK);
typedef asmlinkage SCSI_ADAPTER_CONTROL_STATUS (*PHW_STOP_ADAPTER) (PVOID,
		SCSI_ADAPTER_CONTROL_TYPE, PVOID);

typedef struct HW_INITIALIZATION_DATA {
  ULONG  HwInitializationDataSize;
  INTERFACE_TYPE  AdapterInterfaceType;

  ULONG  DeviceExtensionSize;
  ULONG  SrbExtensionSize;
  ULONG  NumberOfAccessRanges;
  BOOLEAN  MapBuffers;
  BOOLEAN  NeedPhysicalAddresses;
  BOOLEAN  TaggedQueueing;
  BOOLEAN  AutoRequestSense;
  BOOLEAN  MultipleRequestPerLu;
  USHORT  VendorIdLength;
  PVOID  VendorId;
  USHORT  DeviceIdLength;
  PVOID  DeviceId;
  PHW_STOP_ADAPTER  HwAdapterControl;
  PHW_BUILDIO  HwBuildIo;
} PACKED HW_INITIALIZATION_DATA, *PHW_INITIALIZATION_DATA;


/*----------------------- Prototypes--------------------*/
#ifndef __LIBRARY__
asmlinkage void DisableControllerInterrupts(IN PVOID DeviceExtension);
#endif /* __LIBRARY__ */

#endif

