//****************************************************************************
//      Linux Host Driver for SiI SATA RAID Adapters
//
// sii_ctype.h
//
// Copyright (c) 2005  Silicon Image.  All rights reserved.
//
// Creator: Vu Le
//
//****************************************************************************

#ifndef _PDCTYP_H_
#define _PDCTYP_H_

#include <linux/linkage.h>

#define _ENABLE_MODE_SENSE
// #define _ENABLE_WRITE_CACHE

#ifdef __x86_64__
# define _WIN64
# define ARCH_64BIT
#endif

/*
 * Define system data types.
 */

typedef long long                      sii_i64;
typedef long                           sii_ilong;
typedef int                            sii_i32;
typedef short                          sii_i16;
typedef char                           sii_i8;

typedef unsigned long long             sii_u64;
typedef unsigned long                  sii_ulong;
typedef unsigned int                   sii_u32;
typedef unsigned short                 sii_u16;
typedef unsigned char                  sii_u8;

#define KSPIN_LOCK                      sii_mutex_t
#define PKEVENT                         sii_i64
#define KMUTEX                          sii_mutex_t

#ifndef NAME_LEN
#define NAME_LEN                       (128)
#endif /* NAME_LEN */

#ifndef DRIVE_SN_LEN
#define DRIVE_SN_LEN                   (20)
#endif /* SN_LEN */

#ifndef DRIVE_MODEL_LEN
#define DRIVE_MODEL_LEN                (42)
#endif /* SN_LEN */

#define ATA_SECTOR_SIZE                 512

#ifndef TRUE
#define TRUE                           (1)
#define true                           (1)
#endif /* TRUE */

#ifndef FALSE
#define FALSE                          (0)
#define false                          (0)
#endif /* FALSE */

#ifndef FAILURE
#define FAILURE                        (0)
#endif /* FAILURE */

#ifndef    NULL
#define    NULL                        (0)
#endif /* NULL */


typedef void                VOID,*PVOID;
typedef sii_i8              CHAR, *PCHAR, *LPCTSTR;
typedef sii_u8              UCHAR, BYTE, *PUCHAR;

typedef sii_i16             SHORT, *PSHORT, CSHORT, *PCSHORT, WCHAR, *PWCHAR;
typedef sii_u16             USHORT, *PUSHORT;

typedef sii_i32             BOOLEAN, boolean, *PBOOLEAN, INT, *PINT;
typedef sii_u32             UINT32, UINT, *PUINT;
typedef sii_u32             UHLONG,ULONG, *PULONG;
typedef sii_i32             LONG, *PLONG;
typedef sii_ulong           POINTER, ULONG_PTR, UINT_PTR;
typedef sii_u64             UINT64, ULONG64, *PUINT64, __int64;

typedef union
{
        struct {
                sii_u32         LowPart;
                sii_u32         HighPart;
        };
        sii_u64         QuadPart;
} LARGE_INTEGER, *PLARGE_INTEGER;

typedef sii_u64         SCSI_PHYSICAL_ADDRESS;

#define UlongToPtr(a)           ((PVOID)(sii_ulong)a)
#define PtrToUlong(a)           ((ULONG)(sii_ulong)a)

#ifndef IN
#define IN
#endif

#ifndef OUT
#define OUT
#endif

// #define DELAY_EXE_TIME                       501

#define NOCONTRL                      (sii_u32)(-1)
#define NOCHAN                        (sii_u32)(-2)
#define NODSK                         (sii_u32)(-3)
#define NORAIDVOL                     (sii_u32)(-4)
#define NOJBODVOL                     (sii_u32)(-5)
#define NOLUN                         (sii_u32)(-6)
#define NOTARGET                      (sii_u32)(-7)
#define NOBUS                         (sii_u32)(-8)
#define NOPORT                        (sii_u32)(-9)
#define NOID                          (sii_u32)(-10)
#define NOINTRLEV                     (sii_u32)(-11)
#define NOINTRMOD                     (sii_u32)(-12)
#define SP_UNINITIALIZED_VALUE        (sii_u32)(-13)
#define STATUS_REVISION_MISMATCH      (sii_u32)(-14)
#define NOWWN                         (sii_u64)(-15)

#define PACKED                  __attribute__((packed))

typedef struct _SRB_IO_CONTROL
{
        ULONG HeaderLength;
        UCHAR Signature[8];
        ULONG Timeout;
        ULONG ControlCode;
        ULONG ReturnCode;
        ULONG Length;
        LONG    ControllerID;   //ControllerID
        UCHAR   PathID;         //PathId
        UCHAR   TargetID;       //TargetId
} PACKED SRB_IO_CONTROL;


#ifdef Si3132                                   // 3132
#define         IDE_SIGN        "SiI3132"
#define         DEVICE_NAME     "sii3132"
#define         CNTRL_ID        0x3132
#define         FUDGE_KEY       0
#elif defined(Si3124)                           // 3124
#define         IDE_SIGN        "SiI3124"
#define         DEVICE_NAME     "sii3124"
#define         CNTRL_ID        0x3124
#define         FUDGE_KEY       0
#elif defined(Si3114)                           // 3114
#define         IDE_SIGN        "SiI3114"
#define         DEVICE_NAME     "sii3114"
#define         CNTRL_ID        0x3114
#define         FUDGE_KEY       0
#else                                           // 3112
#define         IDE_SIGN        "SiI3112"
#define         DEVICE_NAME     "sii3112"
#define         CNTRL_ID        0x3112
#define         FUDGE_KEY       1
#endif

#define MAXADAPTER              8
#define MAX_CHAN_PER_CONTRL     15
#define MAX_CONTROLLER_NUM      8
#define MAX_CHANNEL_QDEPTH      31
#define MAX_CMD_PER_TARGET      (5 * (MAX_CHANNEL_QDEPTH - 1))

#define SII_IOCTL_PATH          DEVICE_NAME
#define SII_IOCTL_MAJOR         0

#endif /* _PDCTYP_H_ */
