//****************************************************************************
//      Linux Host Driver for SiI SATA RAID Adapters
//
// si_Wscsi.h -
//
// Copyright (c) 2005  Silicon Image.  All rights reserved.
// Creator: Vu Le
//
//****************************************************************************
#ifndef _PDCUTILS
#define _PDCUTILS

#include <sii_ctype.h>

// Linux Valid Range of IOCTL send by the Kernel
#define IOCTL_SII_BASE (0xD000)
#define IOCTL_SII_END  (0xD0FF)

extern asmlinkage void 	KeQuerySystemTime	(LARGE_INTEGER *);
extern asmlinkage void     sii_print		(const char *, ...);
extern asmlinkage void     sii_printc		(const char *, ...);
extern asmlinkage void     sii_printd		(const char *, ...);
extern asmlinkage void     DebugMsg		(const char *, ...);
extern asmlinkage void sii_printCdb(char *str, void *Srb);
extern asmlinkage void sii_printDdb(char *str, void *Srb);
extern asmlinkage void*    sii_alloc               (sii_u32 size, sii_u8 v_ok, sii_ulong);
extern asmlinkage void     sii_free                (void *, int s);
extern asmlinkage int      sii_sprintf		(char *, const char *, ...);
extern asmlinkage int      sii_thread_create(int (*func) (void *), void *);
extern asmlinkage void     sii_usleep		(unsigned long);
extern asmlinkage void     sii_panic		(const char *, ...);


struct sii_mutex
{
    spinlock_t          	sm_mtx;
    volatile unsigned long   	sm_flags;
};
                                                                                       
typedef struct sii_mutex sii_mutex_t;
                                                                                       
#define sii_mutex_init(p)  {spin_lock_init(&((p)->sm_mtx)); (p)->sm_flags = 0;}
#define sii_mutex_exit(p)  spin_unlock_irqrestore(&((p)->sm_mtx), (p)->sm_flags)
#define sii_mutex_enter(p) spin_lock_irqsave(&((p)->sm_mtx), (p)->sm_flags)

#define sii_cc_assert(label, con)   enum { label = 1 / (con) }

#ifdef SII_DEBUG
#define sii_assert(exp)                                                 \
    if (!(exp)) {                                                       \
        sii_panic("Assertion point: file: %s, line: %d\n",              \
                    __FILE__, __LINE__);                                \
    }
#else
#define sii_assert(exp)
#endif // SII_DEBUG

#endif
