//****************************************************************************
//	Linux Host Driver SATA RAID Adapters
//
// sii_scsi.h - Include file
//
// Copyright (c) 2005  Silicon Image.  All rights reserved.
//
//****************************************************************************
// portions of this header copied from minw w32api headers 
/*
 * srb.h or scsi.h
 *
 * This file is part of the mingw w32api package.
 *
 * Contributors:
 *   Created by Casper S. Hornstrup <chorns@users.sourceforge.net>
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain. You may
 * use, modify or distribute it freely.
 *
 * This code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY. ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED. This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
*/
#ifndef _SII_SCSI
#define _SII_SCSI

#include <asm/types.h>
#include <sii_ctype.h>

#define WMIREG_FLAG_EVENT_ONLY_GUID	1

//
// Scsi Request Block structure
//
typedef struct _SCSI_REQUEST_BLOCK { 
  USHORT  Length; 
  UCHAR  Function; 
  UCHAR  SrbStatus; 
  UCHAR  ScsiStatus; 
  UCHAR  PathId; 
  UCHAR  TargetId; 
  UCHAR  Lun; 
  UCHAR  QueueTag; 
  UCHAR  QueueAction; 
  UCHAR  CdbLength; 
  UCHAR  SenseInfoBufferLength; 
  ULONG  SrbFlags; 
  ULONG  DataTransferLength; 
  ULONG  TimeOutValue; 
  PVOID  DataBuffer; 
  PVOID  SenseInfoBuffer; 
  struct _SCSI_REQUEST_BLOCK  *NextSrb;
 union {
  PVOID  OriginalRequest;
  PVOID  OriginalCmd;
 };
  PVOID  SrbExtension;
  /*
  union {
    ULONG  InternalStatus;
    ULONG  QueueSortKey;
  } DUMMYUNIONNAME;
  */ 
    UCHAR Cdb[16];
    USHORT  use_sg;                 // Number of pieces of scatter-gather
    dma_addr_t		Physical_Addr;
    dma_addr_t		Physical_Addr2;
    PVOID		OrigSRB;	// = -1 if already unmapped
					// = 0  if must map
					// > 0  it is a child SRB do not remap
    PVOID		Internal_cmd;
} PACKED SCSI_REQUEST_BLOCK, *PSCSI_REQUEST_BLOCK, **DPSCSI_REQUEST_BLOCK;

//
// Sense Data Format
//
typedef struct _SENSE_DATA
{
    UCHAR ErrorCode:7;
    UCHAR Valid:1;
    UCHAR SegmentNumber;
    UCHAR SenseKey:4;
    UCHAR Reserved:1;
    UCHAR IncorrectLength:1;
    UCHAR EndOfMedia:1;
    UCHAR FileMark:1;
    UCHAR Information[4];
    UCHAR AdditionalSenseLength;
    UCHAR CommandSpecificInformation[4];
    UCHAR AdditionalSenseCode;
    UCHAR AdditionalSenseCodeQualifier;
    UCHAR FieldReplaceableUnitCode;
    UCHAR SenseKeySpecific[3];
} SENSE_DATA, *PSENSE_DATA;


#ifndef PCI_TYPE0_ADDRESSES
#define PCI_TYPE0_ADDRESSES             6
#endif

#ifndef PCI_TYPE1_ADDRESSES
#define PCI_TYPE1_ADDRESSES             2
#endif

typedef struct _PCI_SLOT_NUMBER {
    union {
        struct {
            ULONG   DeviceNumber:5;
            ULONG   FunctionNumber:3;
            ULONG   Reserved:24;
        } bits;
        ULONG   AsULONG;
    } u;
} PCI_SLOT_NUMBER, *PPCI_SLOT_NUMBER;


typedef struct _PCI_COMMON_CONFIG
{
	USHORT	VendorID;        /* (ro)                 */
	USHORT	DeviceID;        /* (ro)                 */
	USHORT	Command;         /* Device control       */
	USHORT	Status;
	UCHAR	RevisionID;       /* (ro)                 */
	UCHAR	ProgIf;           /* (ro)                 */
	UCHAR	SubClass;         /* (ro)                 */
	UCHAR	BaseClass;        /* (ro)                 */
	UCHAR	CacheLineSize;    /* (ro+)                */
	UCHAR	LatencyTimer;     /* (ro+)                */
	UCHAR	HeaderType;       /* (ro)                 */
	UCHAR	BIST;             /* Built in self test   */

	union
	{
		struct _PCI_HEADER_TYPE_0
		{
			ULONG BaseAddresses[PCI_TYPE0_ADDRESSES];
			ULONG CIS;
			USHORT SubVendorID;
			USHORT SubSystemID;
			ULONG ROMBaseAddress;
			ULONG Reserved2[2];
			UCHAR InterruptLine;    /*                    */
			UCHAR InterruptPin;     /* (ro)               */
			UCHAR MinimumGrant;     /* (ro)               */
			UCHAR MaximumLatency;   /* (ro)               */
		} type0;
		struct _PCI_HEADER_TYPE_2
		{
			ULONG   BaseAddress;
			UCHAR   CapabilitiesPtr;
			UCHAR   Reserved2;
			USHORT  SecondaryStatus;
			UCHAR   PrimaryBusNumber;
			UCHAR   CardbusBusNumber;
			UCHAR   SubordinateBusNumber;
			UCHAR   CardbusLatencyTimer;
			ULONG   MemoryBase0;
			ULONG   MemoryLimit0;
			ULONG   MemoryBase1;
			ULONG   MemoryLimit1;
			USHORT  IOBase0_LO;
			USHORT  IOBase0_HI;
			USHORT  IOLimit0_LO;
			USHORT  IOLimit0_HI;
			USHORT  IOBase1_LO;
			USHORT  IOBase1_HI;
			USHORT  IOLimit1_LO;
			USHORT  IOLimit1_HI;
			UCHAR   InterruptLine;
			UCHAR   InterruptPin;
			USHORT  BridgeControl;
			USHORT  SubVendorID;
			USHORT  SubSystemID;
			ULONG   LegacyBaseAddress;
			UCHAR   Reserved3[56];
			ULONG   SystemControl;
			UCHAR   MultiMediaControl;
			UCHAR   GeneralStatus;
			UCHAR   Reserved4[2];
			UCHAR   GPIO0Control;
			UCHAR   GPIO1Control;
			UCHAR   GPIO2Control;
			UCHAR   GPIO3Control;
			ULONG   IRQMuxRouting;
			UCHAR   RetryStatus;
			UCHAR   CardControl;
			UCHAR   DeviceControl;
			UCHAR   Diagnostic;
		} type2;


	} u;
	UCHAR	DeviceSpecific[192];
} PCI_COMMON_CONFIG, *PPCI_COMMON_CONFIG;

#ifndef FIELD_OFFSET
#define FIELD_OFFSET(type, field)    ((LONG)&(((type *)0)->field))
#endif

#ifndef PCI_COMMON_HDR_LENGTH
#define PCI_COMMON_HDR_LENGTH (FIELD_OFFSET (PCI_COMMON_CONFIG, DeviceSpecific))
#endif

// Definition of time interval used for timer call back
// per second = 100 ( 1 = 10ms) is provided in based driver
#define TIME_10_SECONDS		10000000  // rebuild rate
#define TIME_10_MSECONDS	10000  	  // rebuild rate			
#define TIME_2_SECONDS		2000000   // rebuild rate
#define IDLE_TIME_IN_SECONDS    3000000  // about 6 seconds (3 rebuild counts = 6 - 8 seconds) 
#define TIME_750_MILLI_SEC	750000
#define TIME_500_MILLI_SEC	500000   
#define TIME_50_MILLI_SEC	50000

#define DELAY_EXE_TIME          10001

#define WIN_SMART		   	0xB0
/* WIN_SMART sub-commands */
#define SMART_READ_VALUES		0xD0
#define SMART_READ_THRESHOLDS		0xD1
#define SMART_AUTOSAVE			0xD2
#define SMART_SAVE			0xD3
#define SMART_IMMEDIATE_OFFLINE		0xD4
#define SMART_READ_LOG_SECTOR		0xD5
#define SMART_WRITE_LOG_SECTOR		0xD6
#define SMART_WRITE_THRESHOLDS		0xD7
#define SMART_ENABLE			0xD8
#define SMART_DISABLE			0xD9
#define SMART_STATUS			0xDA
#define SMART_AUTO_OFFLINE		0xDB

//
// SRB Status
//
#define SRB_STATUS_PENDING                  0x00
#define SRB_STATUS_SUCCESS                  0x01
#define SRB_STATUS_ABORTED                  0x02
#define SRB_STATUS_ABORT_FAILED             0x03
#define SRB_STATUS_ERROR                    0x04
#define SRB_STATUS_BUSY                     0x05
#define SRB_STATUS_INVALID_REQUEST          0x06
#define SRB_STATUS_INVALID_PATH_ID          0x07
#define SRB_STATUS_NO_DEVICE                0x08
#define SRB_STATUS_TIMEOUT                  0x09
#define SRB_STATUS_SELECTION_TIMEOUT        0x0A
#define SRB_STATUS_COMMAND_TIMEOUT          0x0B
#define SRB_STATUS_MESSAGE_REJECTED         0x0D
#define SRB_STATUS_BUS_RESET                0x0E
#define SRB_STATUS_PARITY_ERROR             0x0F
#define SRB_STATUS_REQUEST_SENSE_FAILED     0x10
#define SRB_STATUS_NO_HBA                   0x11
#define SRB_STATUS_DATA_OVERRUN             0x12
#define SRB_STATUS_UNEXPECTED_BUS_FREE      0x13
#define SRB_STATUS_PHASE_SEQUENCE_FAILURE   0x14
#define SRB_STATUS_BAD_SRB_BLOCK_LENGTH     0x15
#define SRB_STATUS_REQUEST_FLUSHED          0x16
#define SRB_STATUS_INVALID_LUN              0x20
#define SRB_STATUS_INVALID_TARGET_ID        0x21
#define SRB_STATUS_BAD_FUNCTION             0x22
#define SRB_STATUS_ERROR_RECOVERY           0x23
#define SRB_STATUS_NOT_POWERED              0x24

//
// SRB Status Masks
//
#define SRB_STATUS_QUEUE_FROZEN             0x40
#define SRB_STATUS_AUTOSENSE_VALID          0x80
#define SRB_STATUS(Status) (Status & ~(SRB_STATUS_AUTOSENSE_VALID | SRB_STATUS_QUEUE_FROZEN))


//
// SRB Flag Bits
//
#define SRB_FLAGS_QUEUE_ACTION_ENABLE       0x00000002
#define SRB_FLAGS_DISABLE_DISCONNECT        0x00000004
#define SRB_FLAGS_DISABLE_SYNCH_TRANSFER    0x00000008
                                                                                
#define SRB_FLAGS_BYPASS_FROZEN_QUEUE       0x00000010
#define SRB_FLAGS_DISABLE_AUTOSENSE         0x00000020
#define SRB_FLAGS_DATA_IN                   0x00000040
#define SRB_FLAGS_DATA_OUT                  0x00000080
#define SRB_FLAGS_NO_DATA_TRANSFER          0x00000000
#define SRB_FLAGS_UNSPECIFIED_DIRECTION      (SRB_FLAGS_DATA_IN | SRB_FLAGS_DATA_OUT)
                                                                                
#define SRB_FLAGS_NO_QUEUE_FREEZE           0x00000100
#define SRB_FLAGS_ADAPTER_CACHE_ENABLE      0x00000200
#define SRB_FLAGS_FREE_SENSE_BUFFER         0x00000400
                                                                                
#define SRB_FLAGS_IS_ACTIVE                 0x00010000
#define SRB_FLAGS_ALLOCATED_FROM_ZONE       0x00020000
#define SRB_FLAGS_SGLIST_FROM_POOL          0x00040000
#define SRB_FLAGS_BYPASS_LOCKED_QUEUE       0x00080000
                                                                                
#define SRB_FLAGS_NO_KEEP_AWAKE             0x00100000
#define SRB_FLAGS_PORT_DRIVER_ALLOCSENSE    0x00200000
#define SRB_FLAGS_PORT_DRIVER_SENSEHASPORT  0x00400000
#define SRB_FLAGS_DONT_START_NEXT_PACKET    0x00800000
                                                                                
#define SRB_FLAGS_PORT_DRIVER_RESERVED      0x0F000000
#define SRB_FLAGS_CLASS_DRIVER_RESERVED     0xF0000000


/* SRB_FUNCTION_ */
#define SRB_FUNCTION_EXECUTE_SCSI           0x00
#define SRB_FUNCTION_CLAIM_DEVICE           0x01
#define SRB_FUNCTION_IO_CONTROL             0x02
#define SRB_FUNCTION_RECEIVE_EVENT          0x03
#define SRB_FUNCTION_RELEASE_QUEUE          0x04
#define SRB_FUNCTION_ATTACH_DEVICE          0x05
#define SRB_FUNCTION_RELEASE_DEVICE         0x06
#define SRB_FUNCTION_SHUTDOWN               0x07
#define SRB_FUNCTION_FLUSH                  0x08
#define SRB_FUNCTION_ABORT_COMMAND          0x10
#define SRB_FUNCTION_RELEASE_RECOVERY       0x11
#define SRB_FUNCTION_RESET_BUS              0x12
#define SRB_FUNCTION_RESET_DEVICE           0x13
#define SRB_FUNCTION_TERMINATE_IO           0x14
#define SRB_FUNCTION_FLUSH_QUEUE            0x15
#define SRB_FUNCTION_REMOVE_DEVICE          0x16
#define SRB_FUNCTION_WMI                    0x17
#define SRB_FUNCTION_LOCK_QUEUE             0x18
#define SRB_FUNCTION_UNLOCK_QUEUE           0x19
#define SRB_FUNCTION_RESET_LOGICAL_UNIT     0x20

#endif // _SII_SCSI

