//****************************************************************************
// Linux Host Driver for SiI SATA RAID Adapters
//
// siidriver.h - Include file
//
// Copyright (c) 2005  Silicon Image.  All rights reserved.
// Creator: Vu Le
//
//****************************************************************************

#ifndef _SIIDRV_
#define _SIIDRV_

//----------------------------------------------------------------
#ifdef CONFIG_X86_USE_3DNOW
#undef CONFIG_X86_USE_3DNOW
#endif

#ifndef __KERNEL__
#define __KERNEL__
#endif

#ifndef MODULE
#define MODULE
#endif
//----------------------------------------------------------------


#include <linux/version.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/pci.h>
#ifndef __LIBRARY__
#include <linux/init.h>
#include <linux/blkdev.h>
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <linux/proc_fs.h>
#include <linux/reboot.h>
#include <linux/smp_lock.h>
#include <linux/vmalloc.h>
#include <linux/spinlock.h>
#endif /* __LIBRARY__ */
#include <asm/div64.h>

#include <sii_ctype.h>
#include <sii_linux_utils.h>
#include <sii_list.h>
#include <lnxWrapper.h>

#ifndef __LIBRARY__
#include <scsi/scsi.h>

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))

#include <scsi/scsi_host.h>
#include <scsi/scsi_device.h>
#include <scsi/scsi_cmnd.h>

#ifdef DWG_DEBUG
#include <scsi/scsi_dbg.h>
#endif // DWG_DEBUG

#else /* LINUX_VERSION_CODE */

#include "scsi.h"
#include "hosts.h"
#include "sd.h"

#define scsi_host_template  SHT

#ifndef MODULE_LICENSE
# define MODULE_LICENSE(x)
#endif /* MODULE_LICENSE */

#ifndef MODULE_VERSION
# define MODULE_VERSION(x)
#endif /* MODULE_VERSION */

#ifndef IRQ_RETVAL
typedef void irqreturn_t;
# define IRQ_NONE
# define IRQ_HANDLED
# define IRQ_RETVAL(x)
#endif /* IRQ_RETVAL */

#endif /* LINUX_VERSION_CODE */

#include <sii_scsi.h>
#include <sii_Wscsi.h>
#include <debug.h>
#endif /* __LIBRARY__ */

#if defined(SCSI_HAS_HOST_LOCK)
#define SII_SCSI_HAS_HOST_LOCK 1
#endif


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//include common SiI files
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// #define RAID_SUPPORT_EX
#define CONFIG_RAID_SET
#define REBUILD_RATE

#define READ_ATTRIBUTE_BUFFER_SIZE  512

#define READ_ATTRIBUTES                 SMART_READ_VALUES
#define READ_THRESHOLDS                 SMART_READ_THRESHOLDS
#define ENABLE_DISABLE_AUTOSAVE         SMART_AUTOSAVE
#define SAVE_ATTRIBUTE_VALUES           SMART_SAVE
#define EXECUTE_OFFLINE_DIAGS           SMART_IMMEDIATE_OFFLINE
#define RETURN_SMART_STATUS             SMART_STATUS
#define ENABLE_DISABLE_AUTO_OFFLINE     SMART_AUTO_OFFLINE
#define ENABLE_SMART                    SMART_ENABLE
#define DISABLE_SMART                   SMART_DISABLE

/* DO NOT CHANGE THESE VALUES */
#define MAX_SG_TABLE_ENTRIES    (33)
#define MAX_SG_ENTRIES_PER_IO   (MAX_SG_TABLE_ENTRIES - 1)
#define MAX_XFER_SIZE           (128)
/* endof DO NOT CHANGE THESE VALUES */

typedef struct sii_gltab
{
    sii_u8              gl_irq_sharing;
    void*               gl_contrl[MAXADAPTER];
    void*               gl_hosts[MAXADAPTER];
    sii_u32             gl_dev_num;    // number of chips
    sii_u32             gl_chan_num;   // number channel of all chips
    sii_u32             gl_drv_num;    // number drives on all channels.
    sii_u32             gl_drv_maj;    // number drives on all channels.
}PACKED  sii_gltab_t;

extern sii_gltab_t gltab;

asmlinkage ULONG LSPApi(int apin, void **contrl, void *arg1, void *arg2);
asmlinkage ULONG LSPApi2(int apin, void **contrl);

#ifndef __LIBRARY__
/*
 *  Define which interrupt handler needs to be installed
 */
struct sii_driver_ident
{
    sii_u16 vendor;
    sii_u16 device;
    sii_u16 subsystem_vendor;
    sii_u16 subsystem_device;
    int     (*init)(struct pci_dev *pcidev, struct scsi_host_template *);
    char *  name;
    BOOLEAN is_RAID;
    sii_u16 channels;
    int     pci_mask;
#define SII_PCI_MASK_64BITS     1
};

// These for dev_flag in the sii_controller_t
#define SII_INIT        0x00
#define SII_END         0x01
#define SII_HOTPLUG     0x02
#define SII_NXTIO       0x04

typedef struct sii_request_queue
{
        LIST_ENTRY      srq_link;
        struct scsi_cmnd *      srq_oricmd;
} PACKED sii_request_queue_t;

typedef struct sii_channel_queue
{
        LIST_ENTRY      io_chan_link;
        sii_u32         io_chan_count;
} sii_channel_queue_t;

sii_cc_assert(SII_RQ_QUEUE_SZ,
        (sizeof(sii_request_queue_t) <= sizeof(struct scsi_pointer)));

typedef asmlinkage VOID (*PVOID_CB)(IN PVOID DeviceExt);
PVOID sii_RequestTimerCallback(IN PVOID Context, ULONG usecs, PVOID_CB TimerHandler);

typedef struct sii_timer_handle
{
        PVOID_CB        th_callback;
        PVOID           th_para0;
        PVOID           th_para1;
} sii_timer_handle_t;

typedef PORT_CONFIGURATION_INFORMATION sii_port_conf_t;

typedef struct sii_controller
{
    void *hwDefExt; // absolutely must be the first entry in the structure for LSPApi()
#ifdef RAID_INTERFACE
    void *raidDefExt;
#endif
    struct timer_list         siiTimer;
    sii_port_conf_t           dev_port_conf;

    sii_timer_handle_t        dev_timer_handle;

    sii_u32                   ControllerId;     //OFFSET of our adapter
    sii_u32                   RaidSrbExtSize;   //sizeof RAID SRB
    PVOID                     NonCachVirtual;
    dma_addr_t                NonCachPhyscal;
    sii_u32                   NonCachedSize;    // size should be saved as when
                                                // the driver is called in
                                                // Sii_Release(),
    // 3114 SUPPORT  (dynamic values)
    sii_u32  dev_contrl_id;             // e.g. 3112, 3512, 3114, etc
    sii_u32  DEVICES_PER_CHANNEL;       // Max # of devices per channel
    sii_u32  CHANNELS_PER_CONTROLLER;   // Max # of Channels for this controller
    sii_u32  TOTAL_DEVICES;             // Max Grand Total  (devices*channels)
                                        // 3112 has 2 unused devices
    sii_u32  MAX_LUNS;                  // Max Logical Drives
    sii_u32  FUDGE;                     // 3112 valid index for devinfo is 0,2
                                        // (not 0,1) legacy

    // struct tasklet_struct   dev_tasklet;
    // sii_u8          dev_tasklet_running;

    sii_ulong           dev_flag;
    struct completion   dev_io_loop_exited;
    struct semaphore    dev_ioctl_sem;
    struct semaphore    dev_io_sem;
    sii_mutex_t         dev_io_mtx;

    sii_channel_queue_t dev_chan_queue[MAX_CHAN_PER_CONTRL];
    sii_ulong           dev_nxt_io_chan;
    PSCSI_REQUEST_BLOCK dev_free_srb_list;

    struct pci_dev      *dev_pcidev;    // PCI configuration info
    struct Scsi_Host    *dev_host;

#ifdef DWG_ISR_TASKLET
    struct tasklet_struct   isr_tasklet;
#endif // DWG_ISR_TASKLET

    /*
     * For hotplug
     */
    sii_u32             dev_change_dmap[MAX_CHAN_PER_CONTRL];
    sii_u32             dev_active_dmap[MAX_CHAN_PER_CONTRL];

    /*
     * Hook point for HW_DEVICE_EXT struct
     */
    sii_u8      _hw_dev_ext_[0];
} PACKED sii_controller_t;

#define MAX_CMD_PER_CHANNEL(c)  (MAX_CMD_PER_TARGET *           \
                                 ((sii_controller_t*)c)->DEVICES_PER_CHANNEL)

// sii_io.c
int     sii_QueueCommand(struct scsi_cmnd *SCpnt, void (*done)(struct scsi_cmnd *));
int     sii_scsi_scan(sii_controller_t *dev,
                sii_u32 channel, sii_u32 target, int toadd);

void    sii_io_complete_thread_init(sii_controller_t *contrl);

// Device I/O
int sii_do_ioctl(int cmd, void *arg);

PSCSI_REQUEST_BLOCK sii_srb_alloc(sii_controller_t* deviceExt);
void sii_srb_free(DPSCSI_REQUEST_BLOCK dpsrb, sii_controller_t* deviceExt);

asmlinkage void UnMapSGBuffer(sii_controller_t *DeviceExt, PSCSI_REQUEST_BLOCK Srb);
#endif /* __LIBRARY__ */

/* Get offset of field 'y' in structure 'x'  */
#define OFFSET(x, y)    ((sii_ulong)(((char *)&(((x *)0)->y)) -  (char *) 0))
/*
 * ----------------------------------------------------------------------------
 * A macro for getting the outer structure from the address of an
 * element within the structure:
 *
 *  on = outer structure name
 *  en = structure element name
 *  ea = structure element address
 *-----------------------------------------------------------------------------
 */
#define outer(on, en, ea)   ((on *)(((char *)(ea)) - OFFSET(on, en)))

#define FindProblemDeviceEntry                  LinuxFindProblemDeviceEntry

/*/////////////////////////Sii Linux\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
// Define possible status values that can be returned by AbortRequest.
typedef enum _ABORT_STATUS {
        Aborted,
        NotFound,
        InProcess
} ABORT_STATUS, *PABORT_STATUS;


//asmlinkage ABORT_STATUS AbortRequest(IN PVOID Context, IN PVOID Srb);

// Device initialize
asmlinkage BOOLEAN InitializeAdapter(PVOID devExt, PVOID *bassAddresses);
asmlinkage const char * sii_get_device_info(PVOID p, sii_u32 devid);
asmlinkage void sii_dev_ext_init(void *p, UCHAR irq_num, ULONG bus_num, ULONG pcislot_num);
asmlinkage PVOID LinuxFindProblemDeviceEntry(PVOID DeviceExt,
                PVOID devInfo, PVOID CurrentEntry, IN BOOLEAN AllocateFlag);
asmlinkage SCSI_PHYSICAL_ADDRESS LinuxGetPhysicalAddress(PVOID DeviceExt,
               PVOID Srb, PVOID DataPointer, PVOID Length);

asmlinkage void sii_set_srb_bkptr(void *srb);

// Device Timer
asmlinkage void         sii_enable_raid_timer(void*   DeviceExt);
asmlinkage void         sii_disable_raid_timer(void*   DeviceExt);

#endif //ifndef _SIIDRV_
