//****************************************************************************
//      Linux Host Driver for SiI SATA RAID Adapters
//
// siierrno.h - LINUX SCSI low level driver errno
//
// Copyright (c) 2005  Silicon Image.  All rights reserved.
//
// Creator: Vu Le
//
//****************************************************************************

#ifndef SIIERRNO_H
#define SIIERRNO_H

/*
 * SII error codes -- When adding new codes, please update the error code
 *                    to text translate table in siierrno.c
 */
#define SII_PASS         0x00    /* Success */
#define SII_ACTIVE       0x01    /* Status while request is active */
#define SII_FAIL         0x02    /* Generic failure code */
#define SII_OFFLINE      0x03    /* Disk is offline */
#define SII_CRC          0x04    /* CRC error */
#define SII_NOPATH       0x05    /* No path to disk */
#define SII_INVALID      0x06    /* Invalid parm on remote request */
#define SII_SHUTDOWN     0x07    /* system shut down in progress */
#define SII_PATHDOWN     0x08    /* physical disk path is not available */
#define SII_NOMBUF       0x09    /* no message buffer available */
#define SII_RETRY        0x0A    /* should retry request */

#define SII_NOMEMORY     0x0C    /* Out of memory */
#define SII_NOSPACE      0x0D    /* Out of space on device */
#define SII_INUSE        0x1E    /* Resource or device in use */
#define SII_TIMEOUT      0x1F    /* Request timeout (for disk i/o mostly) */
#define SII_DEVBAD       0x10    /* Device has fatal error */
#define SII_HWERR        0x11    /* Any hardware error */
#define SII_DEVRST       0x12    /* The path to the disk is reseting */
#define SII_NOTREADY     0x13    /* The device is not ready to take requests */
#define SII_MEDFAIL      0x14    /* disk media failure */
#define SII_UNITATT      0x15    /* unit attention */
#define SII_ABORTED      0x16    /* Operation aborted */
#define SII_QFULL        0x17    /* Device queue full */

/* Struct for translating error codes to text */
typedef struct textconv
{
    unsigned    tx_code;
    char       *tx_text;

}
textconv_t;

extern textconv_t siixlat[];
extern textconv_t siixlat_short[];

/* Generic code to text lookup function */
extern char *findtext(textconv_t * table, unsigned int code);
extern unsigned findcode(textconv_t * table, char *text);

#endif /* Header guard */
