/*+M*************************************************************************
 * Silicon Image Linux SATA Driver
 *
 * Copyright (c) 2005 Silicon Image, Inc. -- http://www.siliconimage.com
 * All rights reserved.
 *
 *---------------------------------------------------------------------------
 * Developed by TriplePoint, Inc. -- http://www.TriplePoint.com
 *-M*************************************************************************/

/*
 * Version: [Major][Minor][release][patch]
 * Version 1.0.2.0 (Thu Oct 21 16:44:58 PDT 2004):
 *      Base:   32-bit-1.2.0.11
 *      Raid:   None
 *      Wrapper:
 *              -Improve performaces
 *              -IOCTL
 *              -Cleanup all Core dot h files
 * Version 1.0.2.1: (Tue Oct 26 17:09:06 PDT 2004)
 *      Base:   32-bit-1.3.0.12 (Fixing bugs in PM)
 *      Raid:   none
 *      Base:   no new change
 * Version 1.0.3.x: Triple-Point conversion from 2.4 to 2.6
 * Version 1.0.4.1: (Tue Dec 21 15:50:15 PST 2004)
 *      wrapper: 1.0.3.11
 *      Base:   + 32-bit-1.3.0.14 (Fixing bugs in PM)
 *              + fix the do_div()
 *              + Fix MODE_SENSE error
 *              + Fix UDMA vs MDMA problem
 *      Raid:   none
 * Version 1.0.4.2: (Wed Dec 22 14:57:38 PST 2004)
 *      wrapper: 1.0.3.12
 *              +Enable the 64bit build
 *      Base:   no new change
 *      Raid:   none
 * Version 1.0.4.3: (Fri Dec 31 12:25:28 PST 2004)
 *      wrapper: 1.0.3.13
 *              + Fix bugs and build in 64bit-arch.
 *      Base:   no new change (1.3.0.14)
 *      Raid:   none
 * Version 1.0.4.4: (Tue Jan  4 17:09:04 PST 2005)
 *      wrapper: 1.0.3.14
 *              + Fix older RedHat versioned kernel module build. This is
 *              the default build option for RedHat Enterprise 3.2
 *              + Cleanup include to simplify number of includes need by each
 *              of the source files. Most file only require including "siidriver.h"
 *              + Fixing up library source to not use sprintf() directly.
 *              + Handling interrupt out-side the interrupt context using tasklet
 *      Base:   no new change (1.3.0.15)
 *      Raid:   none
 * Version 1.0.4.5: (Thu Jan  6 14:37:33 PST 2005)
 *      wrapper: 1.0.3.14
 *              + Backout the "Handling interrupt out-side the interrupt context using tasklet"
 *      Base:   no new change (1.3.0.15)
 *              + Do not reset while in interrupt.
 *              + With more cleanup and add version string in the generated library.
 *      Raid:   none
 * Version 1.0.4.6: Internel test
 * Version 1.0.4.7: (Tue Jan 11 17:24:03 PST 2005)
 *      wrapper: 1.0.4.17
 *    Updates:
 *    1) Fixup for driver unload on Linux-2.6 to un-register from the SCSI
 *    middle layer before shutting down and releasing the driver resources.
 *    This was causing hangs and complaints from the kernel about SCSI requests
 *    timing out and bad SCSI device state changes.
 *    2) Added call to scsi_device_cancel() during hot un-plug to abort any
 *    outstanding SCSI commands to the device.
 *      Base:   new change (1.3.0.15.1)
 *            a) Re-enable SCSI_MODE_SENSE support to stop kernel messages.
 *                    (_ENABLE_MODE_SENSE)
 *            b) Return write cache disabled from the SCSI_MODE_SENSE so hot un-plug
 *            does not get held up or generate kernel messages. (_ENABLE_WRITE_CACHE)
 *            c) Both options are controlled with ifdef's that are defined or undefined
 *            in the library directory Makefile. _ENABLE_MODE_SENSE should be defined,
 *            and _ENABLE_WRITE_CACHE should NOT be defined.
 *            d) Remove -funit-at-a-time compile flag for Linux-2.6 since it was not
 *            necessary for the build, and will be going away in future GCC releases.
 *            4) Cleanup un-used variable compiler warnings.
 *              + Do not reset while in interrupt.
 *              + With more cleanup and add version string in the generated library.
 *      Raid:   none
 * Version 1.0.4.8: (Wed Jan 19 19:43:47 PST 2005)
 * Version 1.0.4.9: (Wed Jan 19 19:43:47 PST 2005)
 *    Doing pci_map_page instead of pci_map_single
 * Version 1.0.4.10: (Thu Jan 20 16:14:06 PST 2005)
 *    Fix hotplug on live I/O for 64bit bugs   --->Current ship version
 * Version 1.0.4.11: (Thu Feb 01 16:14:06 PST 2005)
 *    Able to build the library anywhere.
 * Version 1.0.4.12: (Thu Feb 01 16:14:06 PST 2005)
 *    Build all controller drivers (3112/3114/3124/3132)
 *      - Just need to change the proj line in both Makefiles
 * Version 1.0.4.13: (Tue Feb 15 12:24:13 PST 2005)
 *    Fix bug in Supp3132.c
 *    Fix naming problem in Rule.make, sii_ctype.h, and siidriver.c
 */

#define SII_MAJOR_NUM           1
#define SII_MINOR_NUM           0
#define SII_SUB_MINOR_NUM       5
#define SII_BUILD_NUM          	8 

#if (defined(Si3112) || defined(Si3114)) 
#define SII_VERSION             "1.0.5.8"
#else
#define SII_VERSION             "1.0.5.8"
#endif

#define SII_DRIVER_VERSION      ((SII_MAJOR_NUM << 24) | \
                                  (SII_MINOR_NUM << 16) | \
                                  (SII_SUB_MINOR_NUM << 8) | \
                                  (SII_BUILD_NUM))
#define SII_DRIVER_BUILD_DATE   __DATE__ " " __TIME__

//#define DWG_TOO_EARLY

const char sii_wrapper_version[] =
	"Silicon Image SATARaid Wrapper Version " SII_VERSION " " SII_DRIVER_BUILD_DATE;

/* Silicon Image ID */
#define SII_PCI_VENDOR_ID       0x1095
#define SII_PCI_VENDOR_ID_ATI   0x1002

/* Silicon Image controller ID */
#define SII_DEVICE_ID3112       0x3112  /* SI_3112 */
#define SII_DEVICE_ID3512       0x3512
#define SII_DEVICE_ID0240       0x0240

#define SII_DEVICE_ID3114       0x3114  /* SI_3114 */
#define SII_DEVICE_ID434E       0x434E
#define SII_DEVICE_ID4e4e       0x4e4e

#define SII_DEVICE_ID3124       0x3124  /* SI_3124 */
#define SII_DEVICE_ID3132       0x3132  /* SI_3132 */

#define sii_ATI_DEVICE_436e         0x436e  // ATI_3112-3512
#define sii_ATI_DEVICE_4379         0x4379  // ATI_3112-3512
#define sii_ATI_DEVICE_437a         0x437a  // ATI_3112-3512

/* Silicon Image Sub ID. */
#define SII_SUB_VENDOR_ID1      0x15D9
#define SII_SUB_VENDOR_ID2      0x1095
#define SII_SUB_VENDOR_ID3      0x8086
#define SII_SUB_VENDOR_ID4      0x1002

/* JBOD */
#define SII_SUB_SYS_ID1         0x3112
#define SII_SUB_SYS_ID2         0x3114
#define SII_SUB_SYS_ID3         0x3423
#define SII_SUB_SYS_ID4         0x3424
#define SII_SUB_SYS_ID5         0x3512
#define SII_SUB_SYS_ID6         0x344E
#define SII_SUB_SYS_ID7         0x3422
#define SII_SUB_SYS_ID8         0x3424

/* RAID */
#define SII_SUB_SYS_ID9         0x6112
#define SII_SUB_SYS_ID10        0x6114
#define SII_SUB_SYS_ID11        0x6124
#define SII_SUB_SYS_ID12        0x6512
#define SII_SUB_SYS_ID13        0x6514

#ifdef Si3132
# define SII_NAME               "SI_3132"
#define SII_DEVICE_ID           SII_DEVICE_ID3132
#elif defined(Si3124)           // 3124
# define SII_NAME               "SI_3124"
#define SII_DEVICE_ID           SII_DEVICE_ID3124
#elif defined(Si3114)           // 3114
# define SII_NAME               "SI_3114"
#define SII_DEVICE_ID           SII_DEVICE_ID3114
#else                           // 3112
#define SII_DEVICE_ID           SII_DEVICE_ID3112
# define SII_NAME               "SI_3112"
#endif

#include <siidriver.h>

asmlinkage void
sii_printc(const char *format, ...);

#undef  CONFIG_SII_PM

// Get pointer to memory allocated for RAID code's extension.
#define PRAID_EXT(DeviceExt) \
	(((PUCHAR)(DeviceExt)) + hwDevExtSz)

#ifdef MODULE
MODULE_AUTHOR("Silicon Image Inc.");
MODULE_DESCRIPTION("SCSI driver for SATARaid controller");
MODULE_LICENSE("Silicon Image");
MODULE_VERSION(SII_VERSION);
#endif /* MODULE */

#ifdef RAID_SUPPORT
extern ULONG driverInitializing;
#else
ULONG driverInitializing = 0;
#endif

extern ULONG hwDevExtSz;
extern int printOff;

sii_gltab_t gltab;
#if 0
static int slaveAlloc(struct scsi_device *ptr);
static int slaveConfig(struct scsi_device *ptr);
static void slaveDestroy(struct scsi_device *ptr);
#endif
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,5,74))
static int sii_detect(struct scsi_host_template *);
static int sii_release(struct Scsi_Host *);
#endif
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
static int sii_proc_info(struct Scsi_Host *, char *, char **, off_t, int, int);
#else
static int sii_proc_info(char *, char **, off_t, int, int, int);
#endif
static const char *sii_info(struct Scsi_Host *);
static int sii_queue_command(struct scsi_cmnd *, void (*done)(struct scsi_cmnd *));
static int sii_abort(struct scsi_cmnd *);
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
static int sii_bios_param(struct scsi_device *, struct block_device *, sector_t, int []);
#else
static int sii_bios_param(struct scsi_disk *, kdev_t, int []);
#endif
static int sii_device_reset(struct scsi_cmnd *);
static int sii_bus_reset(struct scsi_cmnd *);
static int sii_host_reset(struct scsi_cmnd *);

static struct scsi_host_template sii_driver_template = {
    .module                  = THIS_MODULE,
    .name                    = "Silicon Image SCSI to SATA universal host bus adapter",
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,5,74))
    .detect                  = sii_detect,
    .release                 = sii_release,
    .use_new_eh_code         = 1,
#endif
    .proc_name               = DEVICE_NAME, //DWG SII_NAME
    .proc_info               = sii_proc_info,
    .info                    = sii_info,
    .queuecommand            = sii_queue_command,
    .eh_abort_handler        = sii_abort,
    .bios_param              = sii_bios_param,
    //.slave_alloc = slaveAlloc,
    //.slave_configure = slaveConfig,
    //.slave_destroy = slaveDestroy,
    .can_queue               = 0xffff,
    .this_id                 = MAX_CONTROLLER_NUM,
    .sg_tablesize            = MAX_SG_ENTRIES_PER_IO,
    .max_sectors             = MAX_XFER_SIZE, /* max xfer size of 64k */
    .cmd_per_lun             = MAX_CMD_PER_TARGET,
    .eh_device_reset_handler = sii_device_reset,
    .eh_bus_reset_handler    = sii_bus_reset,
    .eh_host_reset_handler   = sii_host_reset,
    .unchecked_isa_dma       = 0,
    .use_clustering          = ENABLE_CLUSTERING,
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,20)) && (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0))
    .highmem_io              = 1,
#endif
    .emulated                = 1, /* not real scsi adapter */
};

static int sii_shutdown(struct notifier_block *this, unsigned long code, void *adapter);
static struct notifier_block sii_reboot_notifier = { sii_shutdown, NULL, 0 };

static struct pci_device_id sii_pci_tbl[] = {
#if 0   // Unused
    { SII_PCI_VENDOR_ID,     SII_DEVICE_ID3112, PCI_ANY_ID, PCI_ANY_ID },
    { SII_PCI_VENDOR_ID,     SII_DEVICE_ID3512, PCI_ANY_ID, PCI_ANY_ID },
    { SII_PCI_VENDOR_ID,     SII_DEVICE_ID3114, PCI_ANY_ID, PCI_ANY_ID },
    { SII_PCI_VENDOR_ID_ATI, PCI_ANY_ID,        PCI_ANY_ID, PCI_ANY_ID },
#endif // Unused
    { SII_PCI_VENDOR_ID,     SII_DEVICE_ID, PCI_ANY_ID, PCI_ANY_ID },
#if ( SII_DEVICE_ID == SII_DEVICE_ID3112 )
    { SII_PCI_VENDOR_ID,     SII_DEVICE_ID3512, PCI_ANY_ID, PCI_ANY_ID },
    { SII_PCI_VENDOR_ID_ATI, sii_ATI_DEVICE_436e, PCI_ANY_ID, PCI_ANY_ID },
    { SII_PCI_VENDOR_ID_ATI, sii_ATI_DEVICE_4379, PCI_ANY_ID, PCI_ANY_ID },
    { SII_PCI_VENDOR_ID_ATI, sii_ATI_DEVICE_437a, PCI_ANY_ID, PCI_ANY_ID },
#endif
    { 0 }
};

MODULE_DEVICE_TABLE(pci, sii_pci_tbl);

asmlinkage ABORT_STATUS
RaidAbortIo(
	IN PVOID	Context,
	IN PVOID	RaidDeviceExtension,
	IN PVOID		Srb
	);
	
asmlinkage UCHAR
RaidShutdownNotification(
	IN PVOID	Context,
	IN PVOID	RaidDeviceExtension,
	IN PVOID		Srb
	);	
#if 0
static int slaveAlloc(struct scsi_device *ptr)
{
    sii_printd("\r\n --- slaveAlloc called, ptr: %x, id: %x, lun: %x, ch: %x\n", 
               ptr, ptr->id, ptr->lun, ptr->channel);
    return 0;
}

static int slaveConfig(struct scsi_device *ptr)
{
    sii_printd("\r\n --- slaveConfig called, ptr: %x, id: %x, lun: %x, ch: %x\n", 
               ptr, ptr->id, ptr->lun, ptr->channel);
    return 0;
}

static void slaveDestroy(struct scsi_device *ptr)
{
    sii_printd("\r\n --- slaveDestroy called, ptr: %x, id: %x, lun: %x, ch: %x\n", 
               ptr, ptr->id, ptr->lun, ptr->channel);
}
#endif
/*+F*************************************************************************
 * Function:
 *   sii_ctl_open
 *
 * Description:
 *   Open the control/configuration interface.
 *-F*************************************************************************/
static int
sii_ctl_open(struct inode * inode_ptr, struct file * file_ptr)
{
    int minor_number;

    /* extract & check the minor number */
    minor_number = MINOR(inode_ptr->i_rdev);
    if (minor_number > (gltab.gl_dev_num - 1))
    {
        WDBG_MSG(("sii_Open: Minor number %d not supported\n", minor_number));
        return -ENODEV;
    }

    return 0;
}

/*+F*************************************************************************
 * Function:
 *   sii_ctl_release
 *
 * Description:
 *   Close/release the control/configuration interface.
 *-F*************************************************************************/
static int
sii_ctl_release(struct inode * inode_ptr, struct file * file_ptr)
{
    return 0;
}

/*+F*************************************************************************
 * Function:
 *   sii_ctl_ioctl
 *
 * Description:
 *   Handle control/configuration interface IOCTL requests.
 *-F*************************************************************************/
static int
sii_ctl_ioctl(struct inode *inode_ptr,
    struct file  *file_ptr, unsigned int cmd, unsigned long arg)
{
    return sii_do_ioctl(cmd, (void *) arg);
}

static struct file_operations sii_fops =
{
    .owner   = THIS_MODULE,
    .open    = sii_ctl_open,
    .release = sii_ctl_release,
    .ioctl   = sii_ctl_ioctl,
};

/*+F*************************************************************************
 * Function:
 *   sii_isr
 *
 * Description:
 *   Handle adapter interrupt requests.
 *-F*************************************************************************/
static irqreturn_t
sii_isr(int irq, void *dev_id, struct pt_regs *regs)
{
    sii_controller_t *contrl;
    int irq_handled;

    if (dev_id == NULL)
    {
        return IRQ_NONE;
    }

    contrl = (sii_controller_t *) dev_id;

#ifdef DWG_ISR_TASKLET
#else // DWG_ISR_TASKLET
    sii_mutex_enter(&contrl->dev_io_mtx);
    irq_handled = LSPApi2(0,(void **)contrl);
    // tasklet_schedule(&contrl->dev_tasklet);
    sii_mutex_exit(&contrl->dev_io_mtx);
#endif // DWG_ISR_TASKLET

    return IRQ_RETVAL(irq_handled);
}

/*+F*************************************************************************
 * Function:
 *   sii_capa_to_cyl
 *
 * Description:
 *   Convert a device capacity to a cylinder count.
 *-F*************************************************************************/
static sii_u32
sii_capa_to_cyl(sii_u32 capa, sii_u32 div)
{
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0))
    sector_div(capa, div);
#else
    capa /= div;
#endif

    return capa;
}

typedef struct sii_DiskParm {
    int   heads;
    int   sectors;
    int   cylinders;
} PACKED sii_DiskParm_t;

/*+F*************************************************************************
 * Function:
 *   sii_bios_param
 *
 * Description:
 *   SCSI entry point to query the device geometry information.
 *-F*************************************************************************/
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
static int
sii_bios_param(struct scsi_device *sdev, struct block_device *bdev,
    sector_t capacity, int geom[])
#else
static int
sii_bios_param(struct scsi_disk *sdev, kdev_t bdev, int geom[])
#endif
{
    sii_DiskParm_t *param = (sii_DiskParm_t*) geom;
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0))
    sii_u32     capacity = sdev->capacity;
#endif

    if (capacity >= (2 * 1024 * 1024))
    {
        if (capacity >= (4 * 1024 * 1024))
        {
            param->heads = 255;
            param->sectors = 63;
        }
        else
        {
            param->heads = 128;
            param->sectors = 32;
        }
    }
    else
    {
        param->heads = 64;
        param->sectors = 32;
    }
    param->cylinders =
        sii_capa_to_cyl(capacity, (param->heads * param->sectors));
WDBG_MSG(("\r\n --- sii_bios_param called - cyl: %x, hds: %x, sctrs: %x\n",
                                param->cylinders, param->heads, param->sectors));
    return 0;
}


/*+F*************************************************************************
 * Function:
 *   sii_proc_info
 *
 * Description:
 *   Driver proc query entry point.
 *-F*************************************************************************/
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
static int
sii_proc_info(struct Scsi_Host *pshost, char *proc_buffer,
    char **start_ptr, off_t offset, int host_no, int write)
#else
static int
sii_proc_info(char *proc_buffer,
    char **start_ptr, off_t offset, int bytes_avail, int host_no, int write)
#endif
{
    int ret;

    if (!proc_buffer)
    {
        return 0;
    }

    if (write || offset > 0)
    {
        return 0;
    }

//DWG Buffer overflow possible
    *start_ptr = proc_buffer;

    ret = sii_sprintf(proc_buffer,
              "SiI Controller %d.%d.%d.%d Build-Date: %s\n",
              SII_DRIVER_VERSION >> 24,
              (SII_DRIVER_VERSION >> 16) & 0xFF,
              (SII_DRIVER_VERSION >> 8) & 0xFF,
              SII_DRIVER_VERSION & 0xFF,
              SII_DRIVER_BUILD_DATE);
WDBG_MSG(("\r\n\n --- sii_proc_info called\n%s",proc_buffer));
    return ret;
}

/*+F*************************************************************************
 * Function:
 *   sii_info
 *
 * Description:
 *   SCSI proc information entry point.
 *-F*************************************************************************/
static const char *
sii_info(struct Scsi_Host *pshost)
{
    static char         info[1024];
    sii_controller_t   *contrl;
    int                 len;
#ifdef DWG_TOO_EARLY
    int                 i;
#endif // DWG_TOO_EARLY

    if (!pshost)
        return NULL;
    contrl = HOSTDATA(pshost);
WDBG_MSG(("\r\n\n --- contrl->TOTAL_DEVICES: %x\n", contrl->TOTAL_DEVICES));
    memset(info, 0, sizeof(info));

    len = sii_sprintf(info, "SiI SATA RAID Controller driver 3x and 6x Series\n"   \
        "      Rev: 0x%x, Date: %s\n"                                \
        "      Memory (unique_id) at 0x%08X,  Irq %d\n",
        SII_DRIVER_VERSION,
        SII_DRIVER_BUILD_DATE,
        pshost->unique_id,
        pshost->irq);

//DWG Buffer overflow possible
#ifdef DWG_TOO_EARLY
    len += sii_sprintf(&info[len], "Device(s) attached: \n");
#if 0
    len += sii_sprintf(&info[len], "    0: not attached\n");
    len += sii_sprintf(&info[len], "    1: not attached\n");
#endif
    for (i=0; i < contrl->TOTAL_DEVICES; i++)
    {
        strcat(info, sii_get_device_info(&contrl->hw_dev_ext[0], i));
    }
#endif // DWG_TOO_EARLY
WDBG_MSG(("\r\n\n --- sii_info called\n"));
#if 0
while (driverInitializing) {
    WDBG_MSG(("driverInitializing\n"));
    udelay(100);
}
udelay(100);
#endif
WDBG_MSG(("\r\n\n --- sii_info initialize complete\n%s",info));
/*
for (i = 1; i < 4; i++) {
    sii_scsi_scan(contrl, i, 0, FALSE);
}
for (i = 1; i < 4; i++) {
    sii_scsi_scan(contrl, i, 0, TRUE);
}
*/
//printOff = 1;


    return info;
}

/*+F*************************************************************************
 * Function:
 *   sii_queue_command
 *
 * Description:
 *   SCSI entry point to queue a new SCSI command to the hardware.
 *-F*************************************************************************/
static int
sii_queue_command(struct scsi_cmnd *SCpnt, void (*done)(struct scsi_cmnd *))
{
    return sii_QueueCommand(SCpnt, done);
}

/*+F*************************************************************************
 * Function:
 *   sii_abort
 *
 * Description:
 *   SCSI entry point to abort the specified SCSI command.
 *-F*************************************************************************/
#ifndef RAID_INTERFACE
ABORT_STATUS AbortRequest(IN PVOID Context, IN PSCSI_REQUEST_BLOCK Srb);
#endif
static int
sii_abort(struct scsi_cmnd *SCpnt)
{
    int status;
    sii_controller_t* contrl = HOSTDATA(SCpnt->device->host);
    PVOID       DeviceExt = &contrl->_hw_dev_ext_[0];

    if (!SCpnt->host_scribble)
    {
        return FAILED;
    }

#ifdef RAID_INTERFACE
    /* Raid Abort */
    status = RaidAbortIo(DeviceExt, PRAID_EXT(DeviceExt),
        (PSCSI_REQUEST_BLOCK) SCpnt->host_scribble);
#else
    /* Base driver abort function */
    status = AbortRequest(DeviceExt,
        (PSCSI_REQUEST_BLOCK) SCpnt->host_scribble);
#endif

    if (status == NotFound)
    {
        return FAILED;
    }
    else if (status == InProcess)
    {
        return FAILED;
    }
    else
    {
        return SUCCESS;
    }
}

/*+F*************************************************************************
 * Function:
 *   sii_device_reset
 *
 * Description:
 *   SCSI device reset entry point.
 *-F*************************************************************************/
static int
sii_device_reset(struct scsi_cmnd *SCpnt)
{
    // sii_print("sii_device_reset ----------\n");
    return 0;
}

/*+F*************************************************************************
 * Function:
 *   sii_bus_reset
 *
 * Description:
 *   SCSI bus reset entry point.
 *-F*************************************************************************/
static int
sii_bus_reset(struct scsi_cmnd *SCpnt)
{
    return 0;
}

/*+F*************************************************************************
 * Function:
 *   sii_host_reset
 *
 * Description:
 *   SCSI host reset entry point.
 *-F*************************************************************************/
static int
sii_host_reset(struct scsi_cmnd *SCpnt)
{
    // sii_print("sii_host_reset ----------\n");
    return 0;
}

/*+F*************************************************************************
 * Function:
 *   sii_shutdown_adapter
 *
 * Description:
 *   Shutdown a running adapter.
 *-F*************************************************************************/
static int
sii_shutdown_adapter(sii_controller_t *contrl)
{
    PSCSI_REQUEST_BLOCK srb = NULL;
    INT                 chan, RaidCnt;
    PVOID               DeviceExt = NULL;

    if (!contrl)
    {
        return (-1);
    }

    RaidCnt =  contrl->RaidSrbExtSize;
    DeviceExt = &contrl->_hw_dev_ext_[0];

    srb = sii_srb_alloc(contrl);
    if (!srb)
    {
        WDBG_MSG(("Shutdown: Srb MALLOC FAILED\n"));
        return -1;
    }

    for (chan = 0; chan < contrl->CHANNELS_PER_CONTROLLER; chan++)
    {
        srb->PathId = chan;
        srb->TargetId = srb->Lun = 0; /* always the master drive for SATA. */
        srb->OriginalRequest = 0;

        // XXX Need to clean this dirty code up.
        // check signal type // shutdown NO MATTER WHAT!
        {
#ifdef RAID_INTERFACE
            UCHAR   status;
            int retry = 4;
            int l;
            do {
                status=RaidShutdownNotification(
                    DeviceExt, PRAID_EXT(DeviceExt), srb);
                    for (l = 0; l < 40; l++) { StallExecution(20000); }
                    barrier();
                    for (l = 0; l < 40; l++) { StallExecution(20000); }
                    barrier();
                    if (!--retry) continue;
                    for (l = 0; l < 40; l++) { StallExecution(20000); }
            } while ((status == SRB_STATUS_PENDING) && retry);
            for (l = 0; l < 40; l++) { StallExecution(20000); }
#endif
        }
    }

    /* Free the srb */
    sii_srb_free(&srb, contrl);

    return NOTIFY_DONE;
}

/*+F*************************************************************************
 * Function:
 *   InitializeData
 *
 * Description:
 *   Initialize and register a new SCSI adapter.
 *-F*************************************************************************/
static int
InitializeData(struct Scsi_Host *pshost, struct pci_dev *pcidev)
{
    sii_controller_t *contrl = NULL;
    INT i;
    BOOLEAN again = FALSE;

    if (!pshost || !pcidev)
    {
        sii_print("Wrong input in InitializeData()\n");
        return FALSE;
    }

    contrl = HOSTDATA(pshost);
    contrl->dev_pcidev      = pcidev;
    contrl->dev_contrl_id   = CNTRL_ID;
    contrl->MAX_LUNS        = 1;
    contrl->FUDGE           = FUDGE_KEY;

    /*
     * Initialize IO completion thread
     */
    contrl->dev_flag = SII_INIT;
    sii_mutex_init(&contrl->dev_io_mtx);
    init_completion(&contrl->dev_io_loop_exited);
    init_MUTEX_LOCKED(&contrl->dev_io_sem);
    init_MUTEX_LOCKED(&contrl->dev_ioctl_sem);
    sii_io_complete_thread_init(contrl);

    /*
     *  initialize the tasklet
     */
    // tasklet_init(&contrl->dev_tasklet, sii_tasklet_do, (unsigned long) contrl);

    /* Add to the end of global tabe list */
    gltab.gl_contrl[gltab.gl_dev_num] = contrl;
    gltab.gl_hosts[gltab.gl_dev_num] = pshost;

    if (SiI_pci_request_regions(pcidev, DEVICE_NAME) < 0)
    {
        sii_print("Could not reserve PCI memory regions\n");
        //return FALSE;
    }

    switch (contrl->dev_contrl_id)
    {
    case 0x3112:
        // This is not the corrected way to prevent collision with
        // open source drivers.
        // XXX Need to re-visit this later: using request_mem_region()
        //MSG("\nInitializeData: Making sure PCI regions are available\n");
        //SiI_pci_release_regions(pcidev);

        // XXX Not sure how 3112 handle 64 bit address mapping!!!
        if (SiI_pci_set_dma_mask(pcidev, (u64) 0xffffffff))
        {
            sii_print("SiI Device No suitable for 64 BIT DMA \n");
            return FALSE;
        }
        break;

    case 0x3114:
        // XXX Not sure how 3114 handle 64 bit address mapping!!!
        if (SiI_pci_set_dma_mask(pcidev, (u64) 0xffffffff))
        {
            sii_print("SiI Device No suitable for 64 BIT DMA \n");
            return FALSE;
        }
        break;

    case 0x3124:
    case 0x3132:
#ifdef ARCH_64BIT
        if (SiI_pci_set_dma_mask(pcidev, (u64) 0xffffffffffffffff))
        {
#endif
            if (SiI_pci_set_dma_mask(pcidev, (u64) 0xffffffff))
            {
                sii_print("SiI Device No suitable for 64 BIT DMA\n");
                return FALSE;
            }
#ifdef ARCH_64BIT
            else
            {
                WDBG_MSG(("!!!!WARNING .... only 32 bit DMA addr!!!!\n"));
                sii_printd("!!!!WARNING .... only 32 bit DMA addr!!!!\n");
            }
        }
#endif
        break;

    default:
        sii_print("Unknown controller ID: %x\n", contrl->dev_contrl_id);
        return FALSE;
    }

#ifdef __i386__
    if (pcidev->resource[PCI_ROM_RESOURCE].start)
    {
        SiI_pci_write_config_byte(pcidev, PCI_ROM_ADDRESS,
            pcidev->resource[PCI_ROM_RESOURCE].start | PCI_ROM_ADDRESS_ENABLE);
        DEBUG("InitializeData: ROM enabled\n");
    }
#endif
    SiI_pci_write_config_byte(pcidev, PCI_LATENCY_TIMER, 64);

    /* Initialize the timer */
    SiI_init_timer(&contrl->siiTimer);
    memset(&contrl->dev_timer_handle, 0, sizeof(contrl->dev_timer_handle));

    SiI_pci_set_master(pcidev);

    if (LSPApi(4, (void **)contrl,
        //NULL, NULL, NULL,
        &contrl->dev_port_conf, &again) != SP_RETURN_FOUND)
    {
        return FALSE;
    }
    contrl->CHANNELS_PER_CONTROLLER   =
        contrl->dev_port_conf.NumberOfBuses;
    contrl->DEVICES_PER_CHANNEL         =
        contrl->dev_port_conf.MaximumNumberOfTargets;
    contrl->TOTAL_DEVICES               =
        contrl->DEVICES_PER_CHANNEL * contrl->CHANNELS_PER_CONTROLLER;



    /* Initialize the controller */
    for (i = 0; i < contrl->CHANNELS_PER_CONTROLLER; i++)
    {
        contrl->dev_chan_queue[i].io_chan_count = 0;
        InitializeListHead(&contrl->dev_chan_queue[i].io_chan_link);
    }
    contrl->dev_nxt_io_chan = NOBUS;

    /* For hotplug */
    //for (i = 0; i < contrl->DEVICES_PER_CHANNEL; i++)
    for (i = 0; i < contrl->CHANNELS_PER_CONTROLLER; i++)
    {
        contrl->dev_change_dmap[i] = 0;
        contrl->dev_active_dmap[i] = ~0;
WDBG_MSG((" --- initData dev_change_dmap: %x, i: %x, dev_active_dmap: %x, i: %x\n",
                   contrl->dev_change_dmap[i], i,  contrl->dev_active_dmap[i], i));
    }

    /* Register our IRQ Handler */
    if (SiI_request_irq(pcidev->irq, sii_isr,
        SA_SHIRQ | SA_SAMPLE_RANDOM | SA_INTERRUPT,
        DEVICE_NAME, contrl) < 0)
    {
        sii_print("Unable to allocate IRQ for SiI-SATA controller.\n");
        return FALSE;
    }

    /* Fill Specific Host Data */
    pshost->irq         = pcidev->irq;
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0))
    pshost->pci_dev     = pcidev;
#endif
    pshost->this_id     = contrl->dev_contrl_id;
    pshost->max_channel = (contrl->CHANNELS_PER_CONTROLLER - 1);
    pshost->max_id      = contrl->DEVICES_PER_CHANNEL;
    pshost->max_lun     = contrl->MAX_LUNS;

    WDBG_MSG(("\r\n --- pshost this_id: %x, max_channel: %x, max_id: %x, max_lun: %x\n",
               pshost->this_id, pshost->max_channel, pshost->max_id, pshost->max_lun));
    return TRUE;
}

/*+F*************************************************************************
 * Function:
 *   sii_create_adapter
 *
 * Description:
 *   Create a new adapter and add it to the system.
 *-F*************************************************************************/
static int
sii_create_adapter(struct pci_dev *pcidev, struct scsi_host_template *tpnt)
{
    //int i;
    struct Scsi_Host *pshost = NULL;
    sii_controller_t *contrl = NULL;

WDBG_MSG(("\r\n\n --- sii_create_adapter\n"));

    if (!pcidev || !tpnt)
    {
        sii_print("sii_create_adapter fail with NULL params\n");
        return -EINVAL;
    }

    if (SiI_pci_enable_device(pcidev))
    {
        sii_print("SiI: failed to enable pci device\n");
        return -EINVAL;
    }
#ifdef RAID_SUPPORT
driverInitializing = 1;
#endif

    if (!LSPApi(0, 0, tpnt, &pshost) || (pshost == NULL))
    {
        sii_print("intial driver load entry failed\n");
        return -EINVAL;
    }
    contrl = HOSTDATA(pshost);
    contrl->hwDefExt = &contrl->_hw_dev_ext_[0];
/*
    if (!DriverEntry(tpnt, &pshost) || (pshost == NULL))
    {
        sii_print("DriverEntry failed\n");
        return -EINVAL;
    }
*/
    /* Initialize the adapter data */
    if (InitializeData(pshost, pcidev) != TRUE)
    {
        //WARN_MSG("Unable to InitializeData\n");
        sii_printd(("Unable to InitializeData\n"));
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0))
        SiI_scsi_unregister(pshost);
#endif
        return -EINVAL;
    }

    //contrl = HOSTDATA(pshost);
#ifdef DWG_ISR_TASKLET
    tasklet_init(&contrl->isr_tasklet, sii_isr_tasklet, (unsigned long) contrl);
#endif // DWG_ISR_TASKLET

    sii_mutex_enter(&contrl->dev_io_mtx);

    if (LSPApi(5, (void **)contrl, 0, 0) != TRUE)
    {
        sii_mutex_exit(&contrl->dev_io_mtx);
        //WARN_MSG("Unable to Initialize\n");
        sii_printd("\r\n --- Unable to Initialize\n");
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0))
        SiI_scsi_unregister(pshost);
#endif
        return -EINVAL;
    }
    sii_mutex_exit(&contrl->dev_io_mtx);
#if 1
if (driverInitializing) {
    WDBG_MSG((" --- driverInitializing\n"));
}
while (driverInitializing) {
    WDBG_MSG(("driverInitializing\n"));
    udelay(100);
}
udelay(100);
    //return -EINVAL;
#endif
    SiI_pci_set_drvdata(pcidev, pshost);

    gltab.gl_dev_num++;

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
    WDBG_MSG((" --- sii_create_adapter calls SiI_scsi_add_host\n"));
    if (SiI_scsi_add_host(pshost, &(pcidev->dev)))
    {
        sii_printd("\r\n --- Add host failed\n");
        SiI_scsi_host_put(pshost);
        return -EINVAL;
    }
//    sii_printd(" --- sii_create_adapter calls SiI_scsi_scan_host\n");
    SiI_scsi_scan_host(pshost);
/*
for (i = 1; i < 4; i++) {
    sii_scsi_scan(contrl, i, 0, FALSE);
}
for (i = 1; i < 4; i++) {
    sii_scsi_scan(contrl, i, 0, TRUE);
}
*/
#else
    pcidev->active = 1;
#endif

    return 0;
}

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,5,74))
/*+F*************************************************************************
 * Function:
 *   sii_detect
 *
 * Description:
 *   Linux-2.4 entry for SCSI adapter detection.
 *-F*************************************************************************/
static int
sii_detect(struct scsi_host_template *tpnt)
{
    struct pci_dev *pcidev;
    int i;
    int status;

WDBG_MSG(("\r\n\n ---- sii_detect called \n"));

    for (i = 0; i < ARRAY_SIZE(sii_pci_tbl); i++)
    {
        pcidev = NULL;
        while ((pcidev = SiI_pci_find_device(
            sii_pci_tbl[i].vendor, sii_pci_tbl[i].device, pcidev)) != NULL)
        {
            status = sii_create_adapter(pcidev, &sii_driver_template);
            if (status < 0)
            {
                continue;
            }
        }
    }

    return gltab.gl_dev_num;
}
#endif /* LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0) */

/*+F*************************************************************************
 * Function:
 *   sii_release
 *
 * Description:
 *   Routine to release the resources of the specified adapter.
 *-F*************************************************************************/
static int
sii_release(struct Scsi_Host *pshost)
{
    sii_controller_t *contrl;
    struct pci_dev *pcidev;
    PSCSI_REQUEST_BLOCK curr;
    INT size;

    if (pshost == NULL)
    {
        sii_print("Can't remove non-initialized adapter\n");
        return FALSE;
    }

    contrl = HOSTDATA(pshost);
    pcidev = contrl->dev_pcidev;
    if (pcidev == NULL)
    {
        sii_print("Can't remove non-initialized controller\n");
        return FALSE;
    }

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
    SiI_scsi_remove_host(pshost);
#endif

    /* Shutdown the adapter */
    sii_shutdown_adapter(contrl);

    SiI_pci_set_drvdata(pcidev, NULL);

    /*
     * Clean up the IO thread
     */
    sii_mutex_enter(&contrl->dev_io_mtx);
    contrl->dev_flag |= SII_END;
    sii_mutex_exit(&contrl->dev_io_mtx);
    up(&contrl->dev_io_sem);

    wait_for_completion(&contrl->dev_io_loop_exited);

#ifdef DWG_ISR_TASKLET
    tasklet_kill(&contrl->isr_tasklet);
#endif // DWG_ISR_TASKLET

    /* Detach adapter interrupts */
    DisableControllerInterrupts(&contrl->_hw_dev_ext_[0]);

    // tasklet_kill(&contrl->dev_tasklet);
    SiI_del_timer(&contrl->siiTimer);

    /* Detach interrupts */
    if (pshost->irq)
    {
        SiI_free_irq(pshost->irq, contrl);
    }

    /* Free the free srb list */
    while (contrl->dev_free_srb_list)
    {
        curr = contrl->dev_free_srb_list;
        contrl->dev_free_srb_list =
            contrl->dev_free_srb_list->NextSrb;
        curr->NextSrb = NULL;
        sii_srb_free(&curr, contrl);
    }

    size = contrl->NonCachedSize;
    SiI_pci_free_consistent(pcidev,
        size, contrl->NonCachVirtual, contrl->NonCachPhyscal);

    SiI_pci_release_regions(pcidev);

    SiI_pci_disable_device(pcidev);

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
    SiI_scsi_host_put(pshost);
#else
    SiI_scsi_unregister(pshost);
#endif

    return TRUE;
}

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
/*+F*************************************************************************
 * Function:
 *   sii_add
 *
 * Description:
 *   PCI subsystem entry point for adding a new adapter.
 *-F*************************************************************************/
static int __devinit
sii_add(struct pci_dev *pcidev, const struct pci_device_id *id)
{
    int status = -ENODEV;

//    sii_printd(" --- sii_add calls sii_create_adapter\n");
WDBG_MSG((" --- sii_add calls sii_create_adapter\n"));

    status = sii_create_adapter(pcidev, &sii_driver_template);

    return status;
}

/*+F*************************************************************************
 * Function:
 *   sii_remove
 *
 * Description:
 *   PCI subsystem entry point for removing an adapter.
 *-F*************************************************************************/
static void __devexit
sii_remove(struct pci_dev *pcidev)
{
    struct Scsi_Host *pshost = NULL;

    pshost = (struct Scsi_Host *) SiI_pci_get_drvdata(pcidev);
    if (pshost == NULL)
    {
        sii_print("Can't remove non-initialized adapter\n");
        return;
    }

    sii_release(pshost);
}

#ifdef CONFIG_SII_PM
/*+F*************************************************************************
 * Function:
 *   sii_suspend
 *
 * Description:
 *   PCI subsystem entry point for adapter suspend.
 *-F*************************************************************************/
static void
sii_suspend(struct pci_dev *pcidev)
{
    SiI_pci_set_power_state(pcidev, 2);
}

/*+F*************************************************************************
 * Function:
 *   sii_resume
 *
 * Description:
 *   PCI subsystem entry point for adapter resume.
 *-F*************************************************************************/
static void
sii_resume(struct pci_dev *pcidev)
{
    SiI_pci_set_power_state(pcidev, 0);
}
#endif /* CONFIG_SII_PM */

static struct pci_driver sii_driver = {
    .name       = SII_NAME,
    .id_table   = sii_pci_tbl,
    .probe      = sii_add,
    .remove     = __devexit_p(sii_remove),
#ifdef CONFIG_SII_PM
    .suspend    = sii_suspend,
    .resume     = sii_resume,
#endif /* CONFIG_SII_PM */
};
#endif /* LINUX_VERSION_CODE */

/*+F*************************************************************************
 * Function:
 *   sii_shutdown
 *
 * Description:
 *   Shutdown/reboot notify entry point.
 *-F*************************************************************************/
static int
sii_shutdown(struct notifier_block *nb, unsigned long event, void *buf)
{
    if ((event != SYS_RESTART) && (event != SYS_HALT) &&
        (event != SYS_POWER_OFF))
    {
        return NOTIFY_DONE;
    }

    unregister_reboot_notifier(&sii_reboot_notifier);

#ifdef DWG_NOTIFY
//DWG
  - tasklet_disable(&sii_tasklet);
  - del_timer()
    foreach (sii_adapter)
    {
        sii_shutdown_adapter(adapter);
    }

    /* Shutdown the adapter */
//DWG Check this since the shutdown needs to shutdown all the adapters.
//    Besides, the buf is not consistently set by the upper layers...
    sii_shutdown_adapter(buf);
#endif /* DWG_NOTIFY */

    return NOTIFY_DONE;
}

/*+F*************************************************************************
 * Function:
 *   sii_init
 *
 * Description:
 *   Load the driver module.
 *-F*************************************************************************/
static int __init
sii_init(void)
{
    int status;

    WDBG_MSG((" --- DRIVER DeviceID == %x\n", SII_DEVICE_ID));   
#ifdef DWG
    if (!SiI_pci_present())
    {
        printk(KERN_ERR SII_NAME": No PCI bus present, module not loaded\n");
        status = -EIO;
        goto e_sii_init;
    }
#endif /* DWG */

    /* Initialize the gl table */
    memset(&gltab, 0, sizeof(gltab));

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
    WDBG_MSG((" --- sii_init calls pci_module_init\n"));
    status = SiI_pci_module_init(&sii_driver);
    WDBG_MSG((" --- sii_init - pci_module_init returns status == %x\n", status));
    if (status == 0) {
        ;
    }
    else
    {
        if (status == -ENODEV)
        {
            printk(KERN_INFO SII_NAME": No adapters found, driver not installed.\n");
            WDBG_MSG(("\r\n --- No adapters found, driver not installed.\n"));
        }
        else
        {
            printk(KERN_INFO SII_NAME": Error %d, driver not installed.\n", status);
            WDBG_MSG(("\r\n --- Error %d, driver not installed.\n", status));
        }

        goto e_sii_init;
    }
#else /* LINUX_VERSION_CODE */
    status = SiI_scsi_register_module(MODULE_SCSI_HA, &sii_driver_template);
    if (status != 0)
    {
        status = -ENODEV;
        goto e_sii_init;
    }
WDBG_MSG(("\r\n ---  SiI_scsi_register_module called\n"));
#endif /* LINUX_VERSION_CODE */

    gltab.gl_drv_maj = register_chrdev(
        SII_IOCTL_MAJOR, SII_IOCTL_PATH,
        (struct file_operations *) &sii_fops);
WDBG_MSG(("\r\n --- register_chrdev\n"));
    if (gltab.gl_drv_maj < 0)
    {
        sii_print("sii: register %s (%d) failed\n",
            SII_IOCTL_PATH, SII_IOCTL_MAJOR);
    }
    else
    {
        sii_print("Major number .. %d\n", gltab.gl_drv_maj);
#ifdef DWG_NOT_ALLOWED
        proc_mknod(SII_IOCTL_PATH, (mode_t) (0777+S_IFCHR),
            (struct proc_dir_entry *) NULL,
            ((kdev_t) MKDEV(gltab.gl_drv_maj ,0)));
#endif /* DWG_NOT_ALLOWED */
    }

    register_reboot_notifier(&sii_reboot_notifier);

e_sii_init:
    return status;
}

/*+F*************************************************************************
 * Function:
 *   sii_exit
 *
 * Description:
 *   Unload the driver module.
 *-F*************************************************************************/
static void __exit
sii_exit(void)
{
    unregister_reboot_notifier(&sii_reboot_notifier);

    unregister_chrdev(gltab.gl_drv_maj, SII_IOCTL_PATH);
#ifdef DWG_NOT_ALLOWED
    remove_proc_entry(SII_IOCTL_PATH, NULL);
#endif /* DWG_NOT_ALLOWED */

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
    SiI_pci_unregister_driver(&sii_driver);
#else /* LINUX_VERSION_CODE */
    SiI_scsi_unregister_module(MODULE_SCSI_HA, &sii_driver_template);
#endif /* LINUX_VERSION_CODE */
}

module_init(sii_init);
module_exit(sii_exit);

