//****************************************************************************
//      Linux Host Driver for SiI SATA RAID Adapters
//
// sii_linux_utils.c - LINUX SCSI low level driver functions
//
// Copyright (c) 2005  Silicon Image.  All rights reserved.
//
// Creator: Vu Le
//
//****************************************************************************
#include <siidriver.h>

#define MAX_KMALLOC (128 * 1024)

extern int printOff;
#if 0
#define do_div64(n,base) ({                     \
 >              u32 __rem;                              \
 >              n = lib_do_div64(n, base, &__rem);      \
 >              __rem; })
#endif
// v_ok: ok to wait go get virtual memory
// ex_flag: more options beside GFP_KERNEL or GFP_ATOMIC
asmlinkage void *
sii_alloc(sii_u32 size, sii_u8 v_ok, sii_ulong eflag)
{
    void    	*p;
    sii_ulong  	flag;

    if (v_ok) {
        flag = GFP_KERNEL;
    } else {
        flag = GFP_ATOMIC;
    }
    flag |= eflag;

    while(1) {
        if (size < MAX_KMALLOC) {
            p = kmalloc(size, flag);
        } else if (v_ok) {
            p = vmalloc(size);
        } else {
            p = NULL;
            break;
        }

        if (p != NULL || !(GFP_KERNEL & __GFP_WAIT)) {
            break;
        }
        sii_usleep(1000);
    }
    if (p) {
        memset(p, 0, size);
    }
    return (p);
}

asmlinkage void
sii_free(void *p, int size)
{
    if (p == NULL)
	return;

    if(size < MAX_KMALLOC) {
        kfree(p);
    } else {
        vfree(p);
    }
}

#if 0
asmlinkage void
sii_print(const char *format, ...)
{
//return;
//    if (printOff) {
//        return;
//    }
    va_list     ap;
    int         total;
    char        print_buffer [250];

    va_start(ap, format);
    total = vsnprintf(print_buffer, sizeof (print_buffer), format, ap);
    va_end(ap);
    printk(print_buffer);
//    udelay(10);
}
#endif

asmlinkage int
sii_sprintf(char *buffer, const char *format, ...)
{
    va_list     ap;
    int         size;

    va_start(ap, format);
    size = vsprintf(buffer, format, ap);
    va_end(ap);
    return (size);
}

static __inline__ LARGE_INTEGER
sii_jiffies_to_nsecs(sii_ulong tick)
{
    struct timespec  ltimespec;
    LARGE_INTEGER    l;

    jiffies_to_timespec(tick, &ltimespec);
    l.QuadPart = ((ltimespec.tv_sec * 1000000) + (ltimespec.tv_nsec / 1000));

    return (l);
}

asmlinkage void
KeQuerySystemTime(LARGE_INTEGER *curtime)
{
    LARGE_INTEGER l;

    l = sii_jiffies_to_nsecs(SiI_jiffies);

    /* System time is in 100's of nanoseconds */
    do_div(l.QuadPart, 100);

    (*curtime).QuadPart = l.QuadPart;
}

static sii_u32
sii_convert_usec2ticks(sii_u32 usecs)
{
        struct timespec  delay;
                                                                                                             
        sii_u32  msecs = (usecs + 999) / 1000;
        delay.tv_sec = msecs / 1000;
        delay.tv_nsec = (msecs % 1000) * 1000000;
        return (timespec_to_jiffies(&delay));
}

asmlinkage void
sii_usleep (sii_ulong usecs)
{
#if 1
    while (usecs > 1000) {
            usecs -= 1000;
            udelay(1000);
        }
	udelay(usecs);
	return;

    if (in_interrupt() || usecs == DELAY_EXE_TIME) {
	while (usecs > 1000) {
            usecs -= 1000;
            udelay(1000);
        }
	udelay(usecs);
    } else {
        sii_ulong ticks = sii_convert_usec2ticks(usecs);
        set_current_state(TASK_INTERRUPTIBLE);
        schedule_timeout(ticks);
    }
#endif
//udelay(usecs);
}

asmlinkage void
sii_panic(const char *format, ...)
{
	char        print_buffer[100];
	va_list     ap;

	va_start(ap, format);
	vsnprintf(print_buffer, sizeof (print_buffer), format, ap);
	va_end(ap);
#if 0
	panic("sii:  %s", print_buffer);
#else
    printk("sii: PANIC: %s\n", print_buffer);
#endif
}

asmlinkage int
sii_thread_create(int (*function) (void *), void *argument)
{
    int                    ktid;

    ktid = kernel_thread(function, argument, CLONE_PARENT);

    /* Setting to hi-prio */
    // sys_setpriority(PRIO_PROCESS, ktid, -5);

    return (ktid);
}

