/* 
 *  print_string.c - Send output to the tty we're running on, regardless if it's
 *  through X11, telnet, etc.  We do this by printing the string to the tty
 *  associated with the current task.
 */
#if 1
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/sched.h>	/* For current */
#include <linux/tty.h>		/* For the tty declarations */
#include <linux/version.h>	/* For LINUX_VERSION_CODE */
#include <sii_scsi.h>
#else
#include <siidriver.h>
#endif

#include <debug.h>

asmlinkage void print_tty(char *str);

asmlinkage void
sii_printc(const char *format, ...);

asmlinkage int
sii_sprintf(char *buffer, const char *format, ...);

asmlinkage void
sii_usleep (sii_ulong usecs);

int printOff = PRINT_OFF;

asmlinkage void Wait(const char *str)
{
	//sii_print("Wait:\n");
	sii_printc(str);
	sii_printc("Wait:\n");
	//printk("sii: Wait: %s\n", str);
	//printc(str);
	//BUG_ON(str == NULL);
	BUG_ON(str != NULL);
	BUG_ON(str == NULL);
//	while (1)
//	{
//		udelay(1000);
//	}
//	BUG_ON(str != NULL);	
}

void __udivdi3(void)
{
	Wait("__udivdi3");
}

asmlinkage void
sii_print(const char *format, ...)
{
//return;
//    if (printOff) {
//        return;
//    }
    va_list     ap;
    int         total;
    char        print_buffer [250];

    va_start(ap, format);
    total = vsnprintf(print_buffer, sizeof (print_buffer), format, ap);
    va_end(ap);
    printk(print_buffer);
#ifdef WRPR_DBG
    print_tty(print_buffer);
#endif
}

#if 1
asmlinkage void
sii_printc(const char *format, ...)
{
    va_list     ap;
    int         total;
    char        print_buffer [250];

    if (printOff) {
        return;
    }

    va_start(ap, format);
    total = vsnprintf(print_buffer, sizeof (print_buffer), format, ap);
    va_end(ap);
    printk(print_buffer);
#ifdef WRPR_DBG
    print_tty(print_buffer);
#endif
}
#endif

asmlinkage void
sii_printd(const char *format, ...)
{
    va_list     ap;
    int         total;
    char        print_buffer [250];

    va_start(ap, format);
    total = vsnprintf(print_buffer, sizeof (print_buffer), format, ap);
    va_end(ap);

    printk(print_buffer);
#ifdef WRPR_DBG
    print_tty(print_buffer);
#endif
    return;
}

#if 1
asmlinkage void
DebugMsg(const char *format, ...)
{
    va_list     ap;
    int         total;
    char        print_buffer [250];

    if (printOff) {
        return;
    }

    va_start(ap, format);
    total = vsnprintf(print_buffer, sizeof (print_buffer), format, ap);
    va_end(ap);
    printk(print_buffer);
#ifdef WRPR_DBG
    print_tty(print_buffer);
#endif
}
#endif


asmlinkage void
sii_printCdb(char *str, PSCSI_REQUEST_BLOCK Srb)
{
    if (printOff) {
        return;
    }
    if (!str) {
        sii_printc(" --- sii_printCdb NULL (str) \n");
    }
    else
    {
        sii_printc(str);
    }
	if (!Srb)
	{
		sii_printc("NULL Srb\n");
		return;
	}	
	sii_printc("Srb: %x, PTL: %x%x%x, SrbStatus %x, cdb: %x %x %x %x %x %x %x %x %x %x\n",
    Srb, Srb->PathId, Srb->TargetId, Srb->Lun,
	Srb->SrbStatus,
	Srb->Cdb[0],
	Srb->Cdb[1],
	Srb->Cdb[2],
	Srb->Cdb[3],
	Srb->Cdb[4],
	Srb->Cdb[5],
	Srb->Cdb[6],
	Srb->Cdb[7],
	Srb->Cdb[8],
	Srb->Cdb[9]);
}

asmlinkage void
sii_printDdb(char *str, PSCSI_REQUEST_BLOCK Srb)
{
//    return;
    int pf = printOff;
    printOff = 0;
    sii_printCdb(str,Srb);
    printOff = pf;
    return;
}



asmlinkage void print_tty(char *str)
#if ( LINUX_VERSION_CODE < KERNEL_VERSION(2,6,5) )
{
return;
}
#else
{
	struct tty_struct *my_tty;

	/* 
	 * tty struct went into signal struct in 2.6.6 
	 */
#if ( LINUX_VERSION_CODE < KERNEL_VERSION(2,6,5) )
	/* 
	 * The tty for the current task 
	 */
	my_tty = current->tty;
#else
	/* 
	 * The tty for the current task, for 2.6.6+ kernels 
	 */
	my_tty = current->signal->tty;
#endif

	/* 
	 * If my_tty is NULL, the current task has no tty you can print to 
	 * (ie, if it's a daemon).  If so, there's nothing we can do.
	 */
	if (my_tty != NULL) {

		/* 
		 * my_tty->driver is a struct which holds the tty's functions,
		 * one of which (write) is used to write strings to the tty. 
		 * It can be used to take a string either from the user's or 
		 * kernel's memory segment.
		 *
		 * The function's 1st parameter is the tty to write to,
		 * because the same function would normally be used for all 
		 * tty's of a certain type.  The 2nd parameter controls 
		 * whether the function receives a string from kernel
		 * memory (false, 0) or from user memory (true, non zero). 
		 * BTW: this param has been removed in Kernels > 2.6.9
		 * The (2nd) 3rd parameter is a pointer to a string.
		 * The (3rd) 4th parameter is the length of the string.
		 *
		 * As you will see below, sometimes it's necessary to use
		 * preprocessor stuff to create code that works for different
		 * kernel versions. The (naive) approach we've taken here 
		 * does not scale well. The right way to deal with this 
		 * is described in section 2 of 
		 * linux/Documentation/SubmittingPatches
		 */
		((my_tty->driver)->write) (my_tty,	/* The tty itself */
#if ( LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,9) )		
					   0,	/* Don't take the string 
						   from user space        */
#endif
					   str,	/* String                 */
					   strlen(str));	/* Length */

		/* 
		 * ttys were originally hardware devices, which (usually) 
		 * strictly followed the ASCII standard.  In ASCII, to move to
		 * a new line you need two characters, a carriage return and a
		 * line feed.  On Unix, the ASCII line feed is used for both 
		 * purposes - so we can't just use \n, because it wouldn't have
		 * a carriage return and the next line will start at the
		 * column right after the line feed.
		 *
		 * This is why text files are different between Unix and 
		 * MS Windows.  In CP/M and derivatives, like MS-DOS and 
		 * MS Windows, the ASCII standard was strictly adhered to,
		 * and therefore a newline requirs both a LF and a CR.
		 */

#if ( LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,9) )		
		((my_tty->driver)->write) (my_tty, 0, "\015\012", 2);
#else
		((my_tty->driver)->write) (my_tty, "\015\012", 2);
#endif
	}
}
#endif
/*
*/


