//****************************************************************************
// Linux Host Driver for SiI SATA RAID Adapters
//
// sii_sysif.h - Include file
//
// Copyright (c) 2004  Silicon Image.  All rights reserved.
// Creator: Vu Le
//
//****************************************************************************

#include <siidriver.h>
#include <siierrno.h>
#include <IdeMgmt.h>

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
#define SCSI_DATA_UNKNOWN   (DMA_BIDIRECTIONAL)
#define SCSI_DATA_WRITE     (DMA_TO_DEVICE)
#define SCSI_DATA_READ      (DMA_FROM_DEVICE)
#define SCSI_DATA_NONE      (DMA_NONE)
                                                                                                                       
#define scsi_to_pci_dma_dir(scsi_dir)   ((int)(scsi_dir))
#define scsi_to_sbus_dma_dir(scsi_dir)  ((int)(scsi_dir))
#endif

extern sii_gltab_t gltab;

static sii_u8
Create_IOCTL_SRB(SCSI_REQUEST_BLOCK **pSRB, SRB_IO_CONTROL *hd)
{
    PSCSI_REQUEST_BLOCK	Srb = NULL;
    sii_controller_t	*contrl = NULL;

    if (!hd)
	return (-EINVAL);

    contrl = gltab.gl_contrl[hd->ControllerID];
    if (!contrl)
	return (-EINVAL);

    Srb = sii_srb_alloc(contrl);
    if(!Srb) {
	sii_printd("SiI CreateSRB:MALLOC FAILED\n");
	return -ENOMEM;
    }

    Srb->Function	= SRB_FUNCTION_IO_CONTROL;
    Srb->PathId		= hd->PathID;
    Srb->TargetId	= hd->TargetID;
    //Srb->DataBuffer	= (PVOID)&hd[0];
    //Srb->DataTransferLength = sizeof(SRB_IDE_IOCTL_BLOCK);        // give maximum size
    memcpy(&Srb->Cdb[0], &hd->ControlCode, sizeof(hd->ControlCode));
    Srb->SrbStatus	= SRB_STATUS_PENDING;
    Srb->OrigSRB = (PSCSI_REQUEST_BLOCK)(-1);  // MARK SRB as mapped
    Srb->Internal_cmd = &contrl->dev_ioctl_sem;
    *pSRB = Srb;
    return (0);
}

static sii_u8
sccmd_ioctl_send(SRB_IDE_IOCTL_BLOCK *pIoBlk, int cmd, void *arg)
{
    SCSI_REQUEST_BLOCK *pSRB = NULL;
    sii_controller_t   *contrl = NULL;
    sii_u8		result = -EINVAL;
    SRB_IO_CONTROL      *hdr = (SRB_IO_CONTROL *)pIoBlk;
    int			i = 8; // 80 ms timeout

    // valid controller IDs start at 1 
    pIoBlk->Header.ControllerID--;
    if (pIoBlk->Header.ControllerID < 0) {
        pIoBlk->Header.ControllerID = 0;
    }

    // buff kernel space buffer
    // arg pointer to user space
    contrl = gltab.gl_contrl[pIoBlk->Header.ControllerID];

    if (!contrl || (result = Create_IOCTL_SRB(&pSRB, hdr)) ){
        sii_printd("Sii sccmd_ioctl_send bad request\n"); 
	return result;
    }

    // Allocate return buffer and copy input data
    pSRB->DataTransferLength  =   hdr->HeaderLength + hdr->Length;
    
    // Data buffer must be accessible by DMA
    pSRB->DataBuffer = sii_alloc (pSRB->DataTransferLength+0x1000,
    			 	  FALSE, GFP_ATOMIC);
   

    // map just in case there is DMA (e.g. Get Identify Data)
    pSRB->use_sg = 0; 
    pSRB->SrbFlags = SRB_FLAGS_DATA_IN + SRB_FLAGS_DATA_OUT;
    pSRB->Physical_Addr = SiI_pci_map_page(contrl->dev_pcidev,
                                pSRB->DataBuffer,
                                (pSRB->DataTransferLength + 0x10000),
                                SiI_scsi_to_pci_dma_dir(pSRB->SrbFlags));

    memcpy(pSRB->DataBuffer,pIoBlk,pSRB->DataTransferLength);	

    //MSG("********** pSRB->DataBuffer %x\n",pSRB->DataBuffer);
    //MSG("pSRB->DataTransferLength  %x\n",pSRB->DataTransferLength);

    result = LSPApi(1, (void **)contrl, pSRB, 0);
    if (result != TRUE) {
            sii_printd("SiI Ioctl: LSPApi returning Error\n"); 
    	    result  = -EINVAL;
	    goto error;
    } else {

      result = pSRB->SrbStatus;

      // Analyze result
      if (result == SRB_STATUS_INVALID_REQUEST) {  

	  // MSG("NON-RAID IOCTL for base driver with command hex=%x\n",cmd);
          // Assume request will complete successfully.
          hdr->ReturnCode = IOCTL_IDE_STATUS_SUCCESS;

      	  switch(cmd)
          {
                case    IOCTL_CMD_MINIPORT_GET_NUM_OF_CNTRLS:
                        pIoBlk->SataCntrlrInfo.ChannelCount = gltab.gl_dev_num;
                        copy_to_user( (void*)arg, (void *)pIoBlk, sizeof(SRB_IDE_IOCTL_BLOCK));
                        result =  SRB_STATUS_SUCCESS; // non-raid can ask for it
			goto exit;

			// base version		    
            	case    IOCTL_CMD_MINIPORT_GET_VERSION:
                        pIoBlk->Version.Major   = IDE_MGMT_MAJOR_VER;
                        pIoBlk->Version.Minor   = IDE_MGMT_MINOR_VER;
                        copy_to_user( (void*)arg, (void *)pIoBlk, sizeof(SRB_IDE_IOCTL_BLOCK));
                        result =  SRB_STATUS_SUCCESS; // non-raid can ask for it
			goto exit;

	    	default: 
        		sii_printd("SiI Ioctl: INVALID REQUEST to base driver stat=%d ctl=%d\n",
				    result, pIoBlk->Header.ControllerID);
        		result  = -EINVAL;
			goto error;
	   }
      } else { 
	  
      		//MSG("SiI Ioctl: RAID DISKS cmd=%d, result=%d SRB=%d ctl=%d\n",
		//		cmd, result, pSRB->SrbStatus,
		//		pIoBlk->Header.ControllerID);

		while ( (pSRB->SrbStatus == SRB_STATUS_PENDING) && i-- ) {
		      StallExecution(10000);
		}

	  	// Convert result
	  	if (pSRB->ScsiStatus && (pSRB->SrbStatus != SRB_STATUS_SUCCESS) ){
        			sii_printd("SiI Ioctl: Bad SCSI status \n"); 
             	    		result = SRB_STATUS_ERROR;  // Scsi Error of some sort
	  	} else {
	      			result = SRB_STATUS_SUCCESS;

              			//return data to user
              		 	if (copy_to_user((void*) arg, pSRB->DataBuffer, 
					 		 pSRB->DataTransferLength)) {
              			    sii_printd("SiI Ioctl: ERROR copy back to user failed\n");
				    result = -EINVAL;
				}

              			//MSG("Returning RAID JNI result=%x\n",result);
          	}
      }
    }
exit:
error:
    //Free memory
    SiI_pci_unmap_page(contrl->dev_pcidev,
                       pSRB->Physical_Addr, pSRB->DataTransferLength+0x10000,
                       SiI_scsi_to_pci_dma_dir(pSRB->SrbFlags));
    sii_free(pSRB->DataBuffer, pSRB->DataTransferLength+0x1000); 

    sii_srb_free(&pSRB, contrl);
    if (result != SRB_STATUS_SUCCESS) {
        //MSG("SiI Ioctl: Returning back IORESULT=%d ctl=%d\n",result,
	//	pIoBlk->Header.ControllerID);
    }

    return (result);
}


int  sii_do_ioctl( int cmd, void * arg ) 
{
    SRB_IDE_IOCTL_BLOCK ioBlock;

    //sii_printd("cmd =%d, arg=%x \n", cmd,arg);

    //  Copy data from user space
    if (copy_from_user( &ioBlock, (void*) arg, sizeof( SRB_IDE_IOCTL_BLOCK))){
       sii_printd("SiI Ioctl: ERROR copy from user failed\n");
       return (-EINVAL);
    }
    return (sccmd_ioctl_send(&ioBlock,cmd, arg));
}
