//****************************************************************************
//      Linux Host Driver for SiI SATA RAID Adapters
//
// sii_timer.c - LINUX SCSI low level driver functions
//
// Copyright (c) 2005  Silicon Image.  All rights reserved.
//
// Creator: Vu Le
//
//****************************************************************************
#include <siidriver.h>

static sii_ulong
convert_usec2ticks(sii_u32 usecs)
{
	struct timespec  delay;

	sii_u32  msecs = (usecs + 999) / 1000;
	delay.tv_sec = msecs / 1000;
        delay.tv_nsec = (msecs % 1000) * 1000000;
        return (timespec_to_jiffies(&delay));
}


void
sii_TimerCallback(void* data)
{
	sii_timer_handle_t	*thp;
	PVOID			DeviceExt = NULL;
	sii_controller_t	*contrl    = NULL;

	if (!data)
		return;

	thp = (sii_timer_handle_t*)data;
	DeviceExt = (PVOID)thp->th_para0;
	
	if (!DeviceExt)
		return;

	contrl = outer(sii_controller_t, _hw_dev_ext_, DeviceExt);
	
	if (!contrl || !thp->th_callback || !thp->th_para0)
		return;

	sii_mutex_enter(&contrl->dev_io_mtx);
	(thp->th_callback)(thp->th_para0);
	sii_mutex_exit(&contrl->dev_io_mtx);
}

PVOID
sii_RequestTimerCallback(IN PVOID Context, ULONG usecs, PVOID_CB TimerHandler)
{
	PVOID	 DeviceExt = (PVOID)Context;
	sii_controller_t *contrl = NULL;
	sii_timer_handle_t *thp = NULL;
	sii_ulong	 ticks = convert_usec2ticks(usecs);

	if (!DeviceExt)
		return (NULL);

	contrl = outer(sii_controller_t, _hw_dev_ext_, DeviceExt);
	if (!contrl)
		return (NULL);

	thp = &contrl->dev_timer_handle;

	if (usecs == 0) {
		//Delete timer
		SiI_del_timer(&contrl->siiTimer);
		sii_disable_raid_timer(DeviceExt);
		return (NULL);
	}

	memset(thp, 0, sizeof (*thp));
	thp->th_callback = TimerHandler;
	thp->th_para0    = Context;		// Always the DeviceExt
	thp->th_para1    = NULL;		// Dont need this


	//
	// Store the pointer to the RAID codes timer handler and start the timer.
	contrl->siiTimer.data		= (sii_ulong)thp;
	contrl->siiTimer.expires	= SiI_jiffies + ticks;
	contrl->siiTimer.function	= (PVOID)sii_TimerCallback;
	sii_enable_raid_timer(DeviceExt);
	SiI_add_timer(&contrl->siiTimer);
	return((PVOID)(&contrl->siiTimer));
}//RequestTimerCallback
