//****************************************************************************
//      Linux Host Driver for SiI SATA RAID Adapters
//
// sii_utils.c - LINUX SCSI low level driver functions
//
// Copyright (c) 2005  Silicon Image.  All rights reserved.
//
// Creator: Vu Le
//
//****************************************************************************
#include <siidriver.h>

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
#define SCSI_DATA_UNKNOWN   (DMA_BIDIRECTIONAL)
#define SCSI_DATA_WRITE     (DMA_TO_DEVICE)
#define SCSI_DATA_READ      (DMA_FROM_DEVICE)
#define SCSI_DATA_NONE      (DMA_NONE)

#define scsi_to_pci_dma_dir(scsi_dir)   ((int)(scsi_dir))
#define scsi_to_sbus_dma_dir(scsi_dir)  ((int)(scsi_dir))
#endif

sii_ulong rphyadrs;


PSCSI_REQUEST_BLOCK
sii_srb_alloc(sii_controller_t *contrl)
{
    sii_u32 RaidCnt;
    PSCSI_REQUEST_BLOCK srb = NULL;

    if (contrl == NULL) {
	sii_print("Error: sii_src_alloc() with NULL contrl\n");
	return (NULL);
    }

    RaidCnt =  contrl->RaidSrbExtSize;
    srb = sii_alloc(
	(sizeof(SCSI_REQUEST_BLOCK) + RaidCnt), FALSE, GFP_ATOMIC);
    if (srb) {
	srb->SrbExtension = ((PUCHAR)srb + sizeof(SCSI_REQUEST_BLOCK));
	sii_set_srb_bkptr(srb);
	return (srb);
    }
    return (NULL);
}

void
sii_srb_free(DPSCSI_REQUEST_BLOCK dpsrb, sii_controller_t* contrl)
{
    PSCSI_REQUEST_BLOCK srb = *dpsrb;
    sii_u32 RaidCnt =  contrl->RaidSrbExtSize;

    sii_free(srb, sizeof(SCSI_REQUEST_BLOCK) + RaidCnt);
    *dpsrb = NULL;
}

asmlinkage VOID
UnMapSGBuffer(sii_controller_t *contrl, PSCSI_REQUEST_BLOCK Srb)
{
	ASSERTMSG("UnMapSGBuffer: NULL inputs\n", ((contrl) || (Srb)))

        if (Srb->OrigSRB == (PSCSI_REQUEST_BLOCK)(1)) {
           //ERROR_MSG("SiI: Page ReUnMapped ");
           WDBG_MSG(("SiI: Page ReUnMapped Failed\n"));
           return;
        }

	// Make sure it unlink from the base
        if(Srb->Physical_Addr && Srb->DataTransferLength && Srb->DataBuffer) {
#if 1
        	INT DataDirection;
                //Resolve data direction
                if(Srb->SrbFlags & SRB_FLAGS_DATA_IN) {
                        DataDirection   = PCI_DMA_FROMDEVICE;
                } else if(Srb->SrbFlags & SRB_FLAGS_DATA_OUT) {
                        DataDirection   = PCI_DMA_TODEVICE;
                } else {
                        DataDirection   = PCI_DMA_BIDIRECTIONAL;
                }
                //Check buffer type
                if(Srb->use_sg) {
                  SiI_pci_unmap_sg(
			contrl->dev_pcidev,
			(struct scatterlist *)Srb->DataBuffer,
			Srb->use_sg, DataDirection);
                } else {
                  SiI_pci_unmap_page(contrl->dev_pcidev,
                       Srb->Physical_Addr, Srb->DataTransferLength+0x10000,
			DataDirection);
                }
#endif
                Srb->OrigSRB = (PSCSI_REQUEST_BLOCK)(1);
        }
        if (Srb->Physical_Addr2) {
            INT DataDirection;
                if(Srb->SrbFlags & SRB_FLAGS_DATA_IN) {
                        DataDirection   = PCI_DMA_FROMDEVICE;
                } else if(Srb->SrbFlags & SRB_FLAGS_DATA_OUT) {
                        DataDirection   = PCI_DMA_TODEVICE;
                } else {
                        DataDirection   = PCI_DMA_BIDIRECTIONAL;
                }
                  SiI_pci_unmap_page(contrl->dev_pcidev,
                       Srb->Physical_Addr2, Srb->SenseInfoBufferLength+0x10000,
                                     DataDirection);
                  //sii_printd(" --- UnmapPa2: %x\n", Srb->Physical_Addr2);
        }
}// UnMapSGBuffer

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0))
#include <asm/uaccess.h>
#endif /* LINUX_VERSION_CODE */

int
sii_scsi_scan(sii_controller_t *dev, sii_u32 channel, sii_u32 target, int toadd)
{
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0))
    struct proc_dir_entry * entry = NULL;
    extern struct proc_dir_entry * proc_scsi;
                                                                                
    if (!dev || channel >= dev->CHANNELS_PER_CONTROLLER ||
	target >= dev->DEVICES_PER_CHANNEL) {
        sii_printd("\r\n --- sii_scsi_scan: Invalid device changemap\n");
        return (-1);
    }
#if 1
sii_printd(" --- scan1 %s #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#\n", __FUNCTION__); //DWG
    sii_printd("%s %s device: controller %d, bus %d, target %d\n",
        __FUNCTION__,
        (toadd)? "add": "remove",
        dev->ControllerId, channel, target);
#endif                                                                                
    if (proc_scsi != (struct proc_dir_entry *)NULL) {
        for (entry = proc_scsi->subdir;
                entry != (struct proc_dir_entry *)NULL; entry = entry->next) {

            if ((entry->low_ino != 0) && (entry->namelen == 4)
                            && (memcmp ("scsi", entry->name, 4) == 0)) {

                if (entry->write_proc !=
                        (int (*)(struct file *,
                                 const char *, unsigned long, void *))NULL) {
                    char buffer[80];
                    int length;
                    mm_segment_t fs;
                                                                                
                    sprintf (buffer,
                        "scsi %s-single-device %d %d %d %d\n",
                        ((toadd) ? "add" : "remove"),
                        dev->dev_host->host_no,
                        channel, target, 0);
                                                                                
                    length = strlen (buffer);
                    fs = get_fs();
                    set_fs(get_ds());
                    length = entry->write_proc( NULL, buffer, length, NULL);
                    set_fs(fs);
                    return (length);
                }
            }
        }
    }
    return (-1);
#else /* LINUX_VERSION_CODE */
    int status = 0;

    if (!dev ||
        (channel >= dev->CHANNELS_PER_CONTROLLER) ||
        (target >= dev->DEVICES_PER_CHANNEL))
    {
        //sii_printd("\r\n --- sii_scsi_scan: Invalid device changemap\n");
        status = -1;
        goto e_sii_scsi_scan;
    }
#if 1
sii_printd(" --- scan2 %s #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#\n", __FUNCTION__); //DWG
    sii_printd("%s %s device: controller %d, bus %d, target %d\n",
        __FUNCTION__,
        (toadd)? "add": "remove",
        dev->ControllerId, channel, target);
#endif
    if (toadd)
    {
        SiI_scsi_add_device(dev->dev_host, channel, target, 0);
    }
    else
    {
        struct scsi_device *sdev;

        sdev = SiI_scsi_device_lookup(dev->dev_host, channel, target, 0);
        if (sdev != NULL)
        {
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,7))
            /* Fixup to allow offline devices to be removed */
            if (sdev->sdev_state != SDEV_RUNNING)
            {
                SiI_scsi_device_set_state(sdev, SDEV_RUNNING);
            }
#endif /* LINUX_VERSION_CODE */

            SiI_scsi_remove_device(sdev);
            SiI_scsi_device_put(sdev);
        }
        else
        {
            sii_print("sii_scsi_scan did not find device!\n");
            status = -1;
            goto e_sii_scsi_scan;
        }
    }

e_sii_scsi_scan:
    return status;
#endif /* LINUX_VERSION_CODE */
}

asmlinkage SCSI_PHYSICAL_ADDRESS
LinuxGetPhysicalAddress(PVOID p, PVOID rq, PVOID DataPointer, PVOID Length)
{
        USHORT                          count;
        struct scatterlist              *sg;
	PSCSI_REQUEST_BLOCK 		Srb = NULL;
	sii_controller_t *contrl = outer(sii_controller_t, _hw_dev_ext_, p);
                                                                                                 
        dma_addr_t                      physicalAddr;
        sii_ulong                      	pa;
        ULONG                           origXferLen;
        PVOID                           origRequestBuff;
	
//sii_printCdb("LGPA", rq);

	if (!p || !rq || !DataPointer) {
		sii_printd("\r\n\n --- LinuxGetPhysicalAddress -- return NULL\n");	
		return ((SCSI_PHYSICAL_ADDRESS)NULL);
	}

	if (!rq && DataPointer == contrl->NonCachVirtual) {
	    return (contrl->NonCachPhyscal);
	}

	Srb	  = (PSCSI_REQUEST_BLOCK)rq;

	if (!Srb->OriginalRequest) {
		if (!Srb->Physical_Addr) {
		    	sii_printd("SiI: Error in LinuxGetPhysicalAddress...! %x\n",Srb->Physical_Addr);
		   	return ((SCSI_PHYSICAL_ADDRESS)NULL);
		} else {
			*(PULONG)Length = 0x8000;   // max length for GUI transfers
			return (Srb->Physical_Addr);
		}
	}
    if (DataPointer == Srb->SenseInfoBuffer) {
        //cdeEnterDebugger();
        if (Srb->Physical_Addr2 == (SCSI_PHYSICAL_ADDRESS)NULL) {
                //cdeEnterDebugger();
                //sii_printDdb(" --- LGPA pa2 Null",Srb);
                Srb->Physical_Addr2 = SiI_pci_map_page(contrl->dev_pcidev,
                           	Srb->SenseInfoBuffer,
				(Srb->SenseInfoBufferLength + 0x10000),
				SiI_scsi_to_pci_dma_dir(Srb->SrbFlags));
                //sii_printd(" --- mapPa2: %x\n", Srb->Physical_Addr2);
        }
        origRequestBuff = Srb->SenseInfoBuffer;
        origXferLen = Srb->SenseInfoBufferLength;
        physicalAddr = Srb->Physical_Addr2;
    }
    else                                                                                             
    {
        if (Srb->Physical_Addr == (SCSI_PHYSICAL_ADDRESS)NULL) {
                Srb->Physical_Addr = SiI_pci_map_page(contrl->dev_pcidev,
                           	Srb->DataBuffer,
				(Srb->DataTransferLength + 0x10000),
				SiI_scsi_to_pci_dma_dir(Srb->SrbFlags));
        }
        origRequestBuff = ((struct scsi_cmnd *) Srb->OriginalRequest)->request_buffer;
        origXferLen = ((struct scsi_cmnd *) Srb->OriginalRequest)->request_bufflen;
        if (DataPointer != origRequestBuff) {
            //cdeEnterDebugger();
        }
        physicalAddr = Srb->Physical_Addr;
    }
                                                                                                 
        // caculate starting offset
        count = DataPointer - origRequestBuff;
                                                                                                 
                                                                                                 
        //initialize sg pointer
        sg = (struct scatterlist *) origRequestBuff;
                                                                                                 
        //
        // check: is it SG list used ?
        //
        if (Srb->use_sg) {
                while (count >= sg_dma_len(sg)) {
                        count -= sg_dma_len(sg);
                        sg++;
                }
                                                                                                 
                physicalAddr = sg_dma_address(sg) + count;
                *(PULONG)Length = sg_dma_len(sg) - count;
        } else {//one big contigious memory
                *(PULONG)Length = origXferLen - count;
                pa = (sii_ulong)(physicalAddr);
                pa = pa + count;
                physicalAddr = (dma_addr_t) pa;
        }
                                                                                                 
        return (physicalAddr);
}//LinuxGetPhysicalAddress

asmlinkage ULONG
LSPInit(
        IN PVOID        argument1,             IN PVOID argument2,
        IN PHW_INITIALIZATION_DATA hw_init,     IN PVOID Optional)
{
    struct scsi_host_template *tpnt = (struct scsi_host_template*)argument1;
    sii_controller_t *contrl = NULL;
    struct Scsi_Host *pshost = NULL;
    sii_port_conf_t  *port_conf = NULL;
    sii_u32          controller_size = 0;

    if (!tpnt || !argument2)
        return (-EINVAL);
//    sii_printd(" --- LSPInit calls SiI_scsi_host_alloc\n");

    controller_size = sizeof(sii_controller_t) + hw_init->DeviceExtensionSize;

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
    pshost = SiI_scsi_host_alloc(tpnt, controller_size);
    if(!pshost) {
        sii_print("Can't allocate SCSI device\n");
        return (-EINVAL);
    }
#else
    pshost = SiI_scsi_register(tpnt, controller_size);
    if(!pshost) {
        sii_print("Can't register SCSI device\n");
        return (-EINVAL);
    }
#endif
    *((struct Scsi_Host **) argument2) = pshost;
    pshost->unique_id = gltab.gl_dev_num;

    contrl = HOSTDATA(pshost);
    memset(contrl, 0, controller_size);
    contrl->dev_host = pshost;
    contrl->hwDefExt = &contrl->_hw_dev_ext_[0];
    contrl->ControllerId = gltab.gl_dev_num;
    contrl->RaidSrbExtSize = hw_init->SrbExtensionSize;

    port_conf = &contrl->dev_port_conf;
    port_conf->Length = sizeof (PORT_CONFIGURATION_INFORMATION);
    port_conf->SystemIoBusNumber = NOBUS;
    port_conf->AdapterInterfaceType = PCIBus;
    port_conf->BusInterruptLevel = NOINTRLEV;
    port_conf->BusInterruptVector = NOINTRLEV;
    port_conf->InterruptMode = NOINTRMOD;
    port_conf->MaximumTransferLength = 0;
    port_conf->NumberOfPhysicalBreaks = SP_UNINITIALIZED_VALUE;
    port_conf->AlignmentMask = 1;	// word aligment
    port_conf->NumberOfAccessRanges = hw_init->NumberOfAccessRanges;
    port_conf->AccessRanges =  NULL;
    port_conf->NumberOfBuses = 0;
    port_conf->MaximumNumberOfTargets = 0;
    return (0);
}

asmlinkage PVOID
LSPGetUncachedExt(
        IN PVOID        DevExt,
        PPORT_CONFIGURATION_INFORMATION configp,
        IN ULONG        size)
{
	void 	*addr = NULL;
	sii_controller_t *contrl = outer(sii_controller_t, _hw_dev_ext_, DevExt);
	contrl->NonCachPhyscal = 0;
        addr = SiI_pci_alloc_consistent(
		contrl->dev_pcidev, size, &contrl->NonCachPhyscal);
	contrl->NonCachVirtual = addr;
	contrl->NonCachedSize = size;
//sii_printd("\r\n --- LSPGetUncachedExt -- ncp: %x, ncv: %x, ncsz: %x\n",
//    contrl->NonCachPhyscal, contrl->NonCachVirtual, contrl->NonCachedSize);
    rphyadrs = contrl->NonCachPhyscal;
	return (addr);
}



asmlinkage BOOLEAN
InitializeAdapter(PVOID p, PVOID *baseAddresses)
{
	sii_controller_t *contrl = outer(sii_controller_t, _hw_dev_ext_, p);
	INT			idx, length;
	POINTER 		page_base;
	POINTER			page_offs;
	POINTER			page_remapped;
//sii_printd(" --- InitializeAdapter - contrl: %x, dev_id: %x, p: %x, *baseAddresses: %x\n", 
  //         contrl, contrl->dev_contrl_id, p, baseAddresses);

	// Map I/O port and memory addresses assigned to controller by the
	// PCI bus driver.
	//linux: get BAR values
//sii_printd(" --- InitializeAdapter - NumberOfAccessRanges: %x\n", 
 //                     contrl->dev_port_conf.NumberOfAccessRanges);
	for(idx = 0; idx < contrl->dev_port_conf.NumberOfAccessRanges; idx++) {
                baseAddresses[idx] = (PVOID) SiI_pci_resource_start(
                                                contrl->dev_pcidev,idx);
                length = SiI_pci_resource_len(contrl->dev_pcidev, idx);
//sii_printd(" --- InitializeAdapter idx: %x, ba: %x\n", idx, baseAddresses[idx]);

		if (((contrl->dev_contrl_id == 0x3114 || contrl->dev_contrl_id == 0x3112) && idx == 5) ||
		    (contrl->dev_contrl_id == 0x3124 && (idx == 0 || idx == 2)) ||
		    (contrl->dev_contrl_id == 0x3132 && (idx == 0 || idx == 2))) {
			page_base = ((POINTER) baseAddresses[idx]) & PAGE_MASK;
			page_offs = (POINTER) baseAddresses[idx] - page_base;
			page_remapped = (POINTER) SiI_ioremap_nocache(
						page_base, page_offs+length);
			baseAddresses[idx] = (PVOID) (page_remapped + page_offs);
//sii_printd(" --- InitializeAdapter idx: %x, ba: %x\n", idx, baseAddresses[idx]);
		}
	}
	return (TRUE);
} // end InitializeAdapter

